from django import forms
from django import test
from django_cradmin import uicontainer

from . import formtest_mixins


class TestFieldBasics(test.TestCase, formtest_mixins.SingleFormRenderableHelperMixin):
    def test_hiddenwidget_sanity(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(widget=forms.HiddenInput()))
        self.assertEqual('hidden', selector.one('input[name="testfield"]')['type'])

    def test_placeholder_default(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField())
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('placeholder'))

    def test_placeholder_kwarg_false(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(),
            fieldwrapper=uicontainer.fieldwrapper.FieldWrapper(
                fieldname='testfield',
                field_renderable=uicontainer.field.Field(placeholder=False)
            ))
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('placeholder'))

    def test_placeholder_kwarg_string(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(),
            fieldwrapper=uicontainer.fieldwrapper.FieldWrapper(
                fieldname='testfield',
                field_renderable=uicontainer.field.Field(placeholder='Type something')
            ))
        self.assertEqual(
            selector.one('input[name="testfield"]')['placeholder'],
            'Type something'
        )

    def test_autofocus_default(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField())
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('autofocus'))

    def test_autofocus_kwarg_false(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(),
            fieldwrapper=uicontainer.fieldwrapper.FieldWrapper(
                fieldname='testfield',
                field_renderable=uicontainer.field.Field(autofocus=False)
            ))
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('autofocus'))

    def test_autofocus_kwarg_true(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(),
            fieldwrapper=uicontainer.fieldwrapper.FieldWrapper(
                fieldname='testfield',
                field_renderable=uicontainer.field.Field(autofocus=True)
            ))
        self.assertTrue(selector.one('input[name="testfield"]').hasattribute('autofocus'))


class TestFieldCharFieldRendering(test.TestCase, formtest_mixins.SingleFormRenderableHelperMixin):
    def test_charfield_name(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField())
        self.assertTrue(selector.exists('input[name="testfield"]'))

    def test_charfield_id(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField())
        self.assertEqual('id_testfield', selector.one('input[name="testfield"]')['id'])

    def test_charfield_type(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField())
        self.assertEqual('text', selector.one('input[name="testfield"]')['type'])

    def test_charfield_required(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField())
        self.assertTrue(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_charfield_not_required(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField(required=False))
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_charfield_widget_attrs(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField(
            widget=forms.TextInput(
                attrs={'my-attribute': '10'}
            )
        ))
        self.assertEqual('10', selector.one('input[name="testfield"]')['my-attribute'])

    def test_charfield_initial(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField(initial='testinitial'))
        self.assertEqual('testinitial', selector.one('input[name="testfield"]')['value'])

    def test_charfield_value_from_form(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(), formvalue='testvalue')
        self.assertEqual('testvalue', selector.one('input[name="testfield"]')['value'])

    def test_css_classes_sanity(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField())
        self.assertEqual(selector.one('input[name="testfield"]').cssclasses_set,
                         {'input', 'test-uicontainer-field', 'test-djangowidget-textinput'})


class TestFieldIntegerFieldRendering(test.TestCase, formtest_mixins.SingleFormRenderableHelperMixin):
    def test_integerfield_name(self):
        selector = self.single_field_formrenderable_htmls(field=forms.IntegerField())
        self.assertTrue(selector.exists('input[name="testfield"]'))

    def test_integerfield_id(self):
        selector = self.single_field_formrenderable_htmls(field=forms.IntegerField())
        self.assertEqual('id_testfield', selector.one('input[name="testfield"]')['id'])

    def test_integerfield_type(self):
        selector = self.single_field_formrenderable_htmls(field=forms.IntegerField())
        self.assertEqual('number', selector.one('input[name="testfield"]')['type'])

    def test_integerfield_required(self):
        selector = self.single_field_formrenderable_htmls(field=forms.IntegerField())
        self.assertTrue(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_integerfield_not_required(self):
        selector = self.single_field_formrenderable_htmls(field=forms.IntegerField(required=False))
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_integerfield_widget_attrs(self):
        selector = self.single_field_formrenderable_htmls(field=forms.IntegerField(
            widget=forms.TextInput(
                attrs={'my-attribute': '10'}
            )
        ))
        self.assertEqual('10', selector.one('input[name="testfield"]')['my-attribute'])

    def test_integerfield_initial(self):
        selector = self.single_field_formrenderable_htmls(field=forms.IntegerField(initial=10))
        self.assertEqual('10', selector.one('input[name="testfield"]')['value'])

    def test_integerfield_value_from_form(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.IntegerField(), formvalue=20)
        self.assertEqual('20', selector.one('input[name="testfield"]')['value'])

    def test_css_classes_sanity(self):
        selector = self.single_field_formrenderable_htmls(field=forms.IntegerField())
        self.assertEqual(selector.one('input[name="testfield"]').cssclasses_set,
                         {'input', 'test-uicontainer-field', 'test-djangowidget-numberinput'})


class TestFieldTextareaRendering(test.TestCase, formtest_mixins.SingleFormRenderableHelperMixin):

    def test_textarea_name(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField(widget=forms.Textarea()))
        self.assertTrue(selector.exists('textarea[name="testfield"]'))

    def test_textarea_id(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField(widget=forms.Textarea()))
        self.assertEqual('id_testfield', selector.one('textarea[name="testfield"]')['id'])

    def test_textarea_required(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField(widget=forms.Textarea()))
        self.assertTrue(selector.one('textarea[name="testfield"]').hasattribute('required'))

    def test_textarea_not_required(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(required=False, widget=forms.Textarea()))
        self.assertFalse(selector.one('textarea[name="testfield"]').hasattribute('required'))

    def test_textarea_widget_attrs(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField(
            widget=forms.Textarea(
                attrs={'my-attribute': '10'}
            )
        ))
        self.assertEqual('10', selector.one('textarea[name="testfield"]')['my-attribute'])

    def test_textarea_initial(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(initial='testinitial', widget=forms.Textarea()))
        self.assertEqual('testinitial', selector.one('textarea[name="testfield"]').alltext)

    def test_textarea_value_from_form(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(widget=forms.Textarea()), formvalue='testvalue')
        self.assertEqual('testvalue', selector.one('textarea[name="testfield"]').alltext)

    def test_css_classes_sanity(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(widget=forms.Textarea()), formvalue='testvalue')
        self.assertEqual(selector.one('textarea[name="testfield"]').cssclasses_set,
                         {'input', 'test-uicontainer-field', 'test-djangowidget-textarea'})


class TestFieldBooleanFieldRendering(test.TestCase, formtest_mixins.SingleFormRenderableHelperMixin):

    def test_booleanfield_name(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField())
        self.assertTrue(selector.exists('input[name="testfield"]'))

    def test_booleanfield_id(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField())
        self.assertEqual('id_testfield', selector.one('input[name="testfield"]')['id'])

    def test_booleanfield_type(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField())
        self.assertEqual('checkbox', selector.one('input[name="testfield"]')['type'])

    def test_booleanfield_required(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField())
        self.assertTrue(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_booleanfield_not_required(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField(required=False))
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_booleanfield_widget_attrs(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField(
            widget=forms.TextInput(
                attrs={'my-attribute': '10'}
            )
        ))
        self.assertEqual('10', selector.one('input[name="testfield"]')['my-attribute'])

    def test_booleanfield_initial_true(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField(initial=True))
        self.assertTrue(selector.one('input[name="testfield"]').hasattribute('checked'))

    def test_booleanfield_initial_false(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField(initial=False))
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('checked'))

    def test_booleanfield_value_from_form_true(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.BooleanField(), formvalue=True)
        self.assertTrue(selector.one('input[name="testfield"]').hasattribute('checked'))

    def test_booleanfield_value_from_form_false(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.BooleanField(), formvalue=False)
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('checked'))

    def test_css_classes_sanity(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField())
        self.assertEqual(selector.one('input[name="testfield"]').cssclasses_set,
                         {'input', 'test-uicontainer-field', 'test-djangowidget-checkboxinput'})


class TestFieldChoiceFieldRendering(test.TestCase, formtest_mixins.SingleFormRenderableHelperMixin):

    def test_choicefield_select_name(self):
        selector = self.single_field_formrenderable_htmls(field=forms.ChoiceField(choices=[('a', 'A')]))
        self.assertTrue(selector.exists('select[name="testfield"]'))

    def test_choicefield_select_id(self):
        selector = self.single_field_formrenderable_htmls(field=forms.ChoiceField(choices=[('a', 'A')]))
        self.assertEqual('id_testfield', selector.one('select[name="testfield"]')['id'])

    def test_choicefield_select_required(self):
        selector = self.single_field_formrenderable_htmls(field=forms.ChoiceField(choices=[('a', 'A')]))
        self.assertTrue(selector.one('select[name="testfield"]').hasattribute('required'))

    def test_choicefield_select_not_required(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A')], required=False))
        self.assertFalse(selector.one('select[name="testfield"]').hasattribute('required'))

    def test_choicefield_select_widget_attrs(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(
                choices=[('a', 'A')],
                widget=forms.Select(
                    attrs={'my-attribute': '10'}
                )))
        self.assertEqual('10', selector.one('select[name="testfield"]')['my-attribute'])

    def test_choicefield_select_initial(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A'), ('b', 'B')],
                                    initial='b'))
        self.assertTrue(selector.one('select[name="testfield"] option[value="b"]').hasattribute('selected'))
        self.assertFalse(selector.one('select[name="testfield"] option[value="a"]').hasattribute('selected'))

    def test_choicefield_select_value_from_form(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A'), ('b', 'B')]),
            formvalue='b')
        self.assertTrue(selector.one('select[name="testfield"] option[value="b"]').hasattribute('selected'))
        self.assertFalse(selector.one('select[name="testfield"] option[value="a"]').hasattribute('selected'))

    def test_css_classes_sanity(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A')]))
        self.assertEqual(selector.one('select[name="testfield"]').cssclasses_set,
                         {'test-uicontainer-field', 'test-djangowidget-select'})

    def test_not_wrapped_in_label(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')]))
        self.assertTrue(selector.exists('label'))
        self.assertTrue(selector.exists('select'))
        self.assertFalse(selector.exists('label select'))


class TestFieldChoiceFieldRadioWidgetRendering(test.TestCase,
                                               formtest_mixins.SingleFormRenderableHelperMixin):
    def test_choicefield_radio_name(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A')],
                                    widget=forms.RadioSelect()))
        self.assertEqual(selector.count('input[name="testfield"]'), 1)

    def test_choicefield_radio_multiple(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A'), ('b', 'B')],
                                    widget=forms.RadioSelect()))
        self.assertEqual(selector.count('input[name="testfield"]'), 2)

    def test_choicefield_radio_id(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A')],
                                    widget=forms.RadioSelect()))
        self.assertEqual('id_testfield_0', selector.one('input[name="testfield"]')['id'])

    def test_choicefield_radio_id_multiple(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A'), ('b', 'B'), ('c', 'C')],
                                    widget=forms.RadioSelect()))
        self.assertTrue(selector.exists('#id_testfield_0'))
        self.assertTrue(selector.exists('#id_testfield_1'))
        self.assertTrue(selector.exists('#id_testfield_2'))

    def test_choicefield_radio_required(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A')],
                                    widget=forms.RadioSelect(),
                                    required=True))
        self.assertTrue(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_choicefield_radio_not_required(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A')],
                                    widget=forms.RadioSelect(),
                                    required=False))
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_choicefield_radio_widget_attrs(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(
                choices=[('a', 'A')],
                widget=forms.RadioSelect(
                    attrs={'my-attribute': '10'}
                )))
        self.assertEqual('10', selector.one('input[name="testfield"]')['my-attribute'])

    def test_choicefield_radio_initial(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A'), ('b', 'B')],
                                    initial='b',
                                    widget=forms.RadioSelect()))
        self.assertFalse(selector.one('#id_testfield_wrapper input[value="a"]').hasattribute('checked'))
        self.assertTrue(selector.one('#id_testfield_wrapper input[value="b"]').hasattribute('checked'))

    def test_choicefield_radio_value_from_form(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A'), ('b', 'B')],
                                    initial='b',
                                    widget=forms.RadioSelect()),
            formvalue='a')
        self.assertTrue(selector.one('#id_testfield_wrapper input[value="a"]').hasattribute('checked'))
        self.assertFalse(selector.one('#id_testfield_wrapper input[value="b"]').hasattribute('checked'))

    def test_choicefield_radio_main_label_no_for(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A')],
                                    widget=forms.RadioSelect()))
        self.assertFalse(selector.one('#id_testfield_label').hasattribute('for'))


class TestFieldMultipleChoiceFieldRendering(test.TestCase,
                                            formtest_mixins.SingleFormRenderableHelperMixin):
    def test_multichoicefield_name(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')]))
        self.assertTrue(selector.exists('select[name="testfield"]'))

    def test_multichoicefield_multiple(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A'), ('b', 'B')]))
        self.assertEqual(selector.count('select[name="testfield"] option'), 2)

    def test_multichoicefield_id(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')]))
        self.assertEqual('id_testfield', selector.one('select[name="testfield"]')['id'])

    def test_multichoicefield_multipe_attribute(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')]))
        self.assertTrue(selector.one('#id_testfield').hasattribute('multiple'))

    def test_multichoicefield_id_multiple(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A'), ('b', 'B'), ('c', 'C')]))
        self.assertEqual(selector.count('#id_testfield option'), 3)

    def test_multichoicefield_required(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')],
                                            required=True))
        self.assertTrue(selector.one('select[name="testfield"]').hasattribute('required'))

    def test_multichoicefield_not_required(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')],
                                            required=False))
        self.assertFalse(selector.one('select[name="testfield"]').hasattribute('required'))

    def test_multichoicefield_widget_attrs(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(
                choices=[('a', 'A')],
                widget=forms.SelectMultiple(
                    attrs={'my-attribute': '10'}
                )))
        self.assertEqual('10', selector.one('select[name="testfield"]')['my-attribute'])

    def test_multichoicefield_initial(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A'), ('b', 'B')],
                                            initial='b'))
        self.assertFalse(selector.one('#id_testfield option[value="a"]').hasattribute('selected'))
        self.assertTrue(selector.one('#id_testfield option[value="b"]').hasattribute('selected'))

    def test_multichoicefield_value_from_form(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A'), ('b', 'B')],
                                            initial='b'),
            formvalue='a')
        self.assertTrue(selector.one('#id_testfield option[value="a"]').hasattribute('selected'))
        self.assertFalse(selector.one('#id_testfield option[value="b"]').hasattribute('selected'))

    def test_multichoicefield_not_wrapped_in_label(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')]))
        self.assertTrue(selector.exists('label'))
        self.assertTrue(selector.exists('select'))
        self.assertFalse(selector.exists('label select'))


class TestFieldMultipleChoiceFieldCheckboxWidgetRendering(test.TestCase,
                                                          formtest_mixins.SingleFormRenderableHelperMixin):
    def test_multichoicefield_checkbox_name(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')],
                                            widget=forms.CheckboxSelectMultiple()))
        self.assertEqual(selector.count('input[name="testfield"]'), 1)

    def test_multichoicefield_checkbox_multiple(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A'), ('b', 'B')],
                                            widget=forms.CheckboxSelectMultiple()))
        self.assertEqual(selector.count('input[name="testfield"]'), 2)

    def test_multichoicefield_checkbox_id(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')],
                                            widget=forms.CheckboxSelectMultiple()))
        self.assertEqual('id_testfield_0', selector.one('input[name="testfield"]')['id'])

    def test_multichoicefield_checkbox_id_multiple(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A'), ('b', 'B'), ('c', 'C')],
                                            widget=forms.CheckboxSelectMultiple()))
        self.assertTrue(selector.exists('#id_testfield_0'))
        self.assertTrue(selector.exists('#id_testfield_1'))
        self.assertTrue(selector.exists('#id_testfield_2'))

    def test_multichoicefield_checkbox_required(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')],
                                            widget=forms.CheckboxSelectMultiple(),
                                            required=True))
        # NOTE: Multiple checkboxes can not have required attribute - they do not
        #       seem to be handled the same way as radio (in html5).
        #       This makes a bit of sense because what we mean by required in Django
        #       with required is that at least one must be checked, and HTML5
        #       required do not seem to use the same logic.
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_multichoicefield_checkbox_not_required(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')],
                                            widget=forms.CheckboxSelectMultiple(),
                                            required=False))
        self.assertFalse(selector.one('input[name="testfield"]').hasattribute('required'))

    def test_multichoicefield_checkbox_widget_attrs(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(
                choices=[('a', 'A')],
                widget=forms.CheckboxSelectMultiple(
                    attrs={'my-attribute': '10'}
                )))
        self.assertEqual('10', selector.one('input[name="testfield"]')['my-attribute'])

    def test_multichoicefield_checkbox_initial(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A'), ('b', 'B')],
                                            initial='b',
                                            widget=forms.CheckboxSelectMultiple()))
        self.assertFalse(selector.one('#id_testfield_wrapper input[value="a"]').hasattribute('checked'))
        self.assertTrue(selector.one('#id_testfield_wrapper input[value="b"]').hasattribute('checked'))

    def test_multichoicefield_checkbox_value_from_form(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A'), ('b', 'B')],
                                            initial='b',
                                            widget=forms.CheckboxSelectMultiple()),
            formvalue='a')
        self.assertTrue(selector.one('#id_testfield_wrapper input[value="a"]').hasattribute('checked'))
        self.assertFalse(selector.one('#id_testfield_wrapper input[value="b"]').hasattribute('checked'))

    def test_multichoicefield_checkbox_main_label_no_for(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.MultipleChoiceField(choices=[('a', 'A')],
                                            widget=forms.CheckboxSelectMultiple()))
        self.assertFalse(selector.one('#id_testfield_label').hasattribute('for'))


class TestHiddenField(test.TestCase, formtest_mixins.SingleFormRenderableHelperMixin):
    def single_field_formrenderable_fieldwrapper_factory(self):
        return uicontainer.fieldwrapper.FieldWrapper(
            fieldname='testfield',
            field_renderable=uicontainer.field.HiddenField())

    def test_charfield_type(self):
        selector = self.single_field_formrenderable_htmls(field=forms.CharField())
        self.assertEqual('hidden', selector.one('input[name="testfield"]')['type'])

    def test_integerfield_type(self):
        selector = self.single_field_formrenderable_htmls(field=forms.IntegerField())
        self.assertEqual('hidden', selector.one('input[name="testfield"]')['type'])

    def test_textarea_type(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.CharField(initial='testinitial', widget=forms.Textarea()))
        self.assertEqual('hidden', selector.one('input[name="testfield"]')['type'])

    def test_booleanfield_type(self):
        selector = self.single_field_formrenderable_htmls(field=forms.BooleanField())
        self.assertEqual('hidden', selector.one('input[name="testfield"]')['type'])

    def test_choicefield_type(self):
        selector = self.single_field_formrenderable_htmls(
            field=forms.ChoiceField(choices=[('a', 'A')]))
        self.assertEqual('hidden', selector.one('input[name="testfield"]')['type'])



import htmls
from django import test
from django_cradmin import uicontainer


class TestButton(test.TestCase):
    def test_sanity(self):
        container = uicontainer.button.Button().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('button'))

    def test_no_text(self):
        container = uicontainer.button.Button().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button').alltext_normalized, '')

    def test_with_text(self):
        container = uicontainer.button.Button(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button').alltext_normalized, 'test')

    def test_button_type_default(self):
        container = uicontainer.button.Button().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['type'], 'button')

    def test_button_type_kwarg(self):
        container = uicontainer.button.Button(button_type='submit').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['type'], 'submit')

    def test_name_default(self):
        container = uicontainer.button.Button().bootstrap()
        selector = htmls.S(container.render())
        self.assertFalse(selector.one('button').hasattribute('name'))

    def test_name_kwarg(self):
        container = uicontainer.button.Button(name='testname').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['name'], 'testname')

    def test_default_css_classes(self):
        container = uicontainer.button.Button().bootstrap()
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['class'], 'button')


class TestButtonPrimary(test.TestCase):
    def test_button_type_default(self):
        container = uicontainer.button.ButtonPrimary().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['type'], 'button')

    def test_default_css_classes(self):
        container = uicontainer.button.ButtonPrimary().bootstrap()
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['class'], 'button  button--primary')


class TestButtonHistoryBack(test.TestCase):
    def test_button_type_default(self):
        container = uicontainer.button.ButtonHistoryBack().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['type'], 'button')

    def test_default_css_classes(self):
        container = uicontainer.button.ButtonHistoryBack().bootstrap()
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['class'], 'button')

    def test_onclick_attribute(self):
        container = uicontainer.button.ButtonHistoryBack().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['onclick'],
                         'history.back();return false;')


class TestSubmit(test.TestCase):
    def test_sanity(self):
        container = uicontainer.button.Submit().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('button'))

    def test_no_text(self):
        container = uicontainer.button.Submit().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button').alltext_normalized, '')

    def test_with_text(self):
        container = uicontainer.button.Submit(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button').alltext_normalized, 'test')

    def test_button_type_default(self):
        container = uicontainer.button.Submit().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['type'], 'submit')

    def test_button_type_kwarg(self):
        container = uicontainer.button.Submit(button_type='button').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['type'], 'button')

    def test_name_default(self):
        container = uicontainer.button.Submit().bootstrap()
        selector = htmls.S(container.render())
        self.assertFalse(selector.one('button').hasattribute('name'))

    def test_name_kwarg(self):
        container = uicontainer.button.Submit(name='testname').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['name'], 'testname')

    def test_default_css_classes(self):
        container = uicontainer.button.Submit().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button').cssclasses_set,
                         {'button', 'test-submit'})


class TestSubmitPrimary(test.TestCase):
    def test_button_type_default(self):
        container = uicontainer.button.SubmitPrimary().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button')['type'], 'submit')

    def test_default_css_classes(self):
        container = uicontainer.button.SubmitPrimary().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('button').cssclasses_set,
                         {'button', 'button--primary', 'test-submit-primary'})



from __future__ import unicode_literals

from django_cradmin.uicontainer import messagescontainer

from . import container
from . import form_mixins
from . import field
from . import label
from . import field_help_text


class FieldWrapper(container.AbstractContainerRenderable, form_mixins.FormRenderableChildMixin):
    """
    Renders a form field.

    This just renders a wrapper ``div``, and lets other renderable classes
    render the field, label, etc.:

    - A :class:`~django_cradmin.uicontainer.label.Label` renders the label.
      This can be overridden using the ``label_renderable`` kwarg, or in
      :meth:`.get_default_label_renderable`.
    - A :class:`~django_cradmin.uicontainer.field.BaseFieldRenderable` renders the
      form field. This can be overridden using the ``field_renderable`` kwarg,
      or in :meth:`.get_default_field_renderable`.
    - A :class:`~django_cradmin.uicontainer.help_text.AutomaticHelpText`
      renders the help text. This can be overridden using the ``help_text_renderable``
      kwarg, or in :meth:`.get_default_help_text_renderable`.
    """
    template_name = 'django_cradmin/uicontainer/fieldwrapper.django.html'

    def __init__(self, fieldname, label_renderable=None, field_renderable=None,
                 help_text_renderable=None, messages_container=None, **kwargs):
        self.fieldname = fieldname
        self.label_renderable = label_renderable or self.get_default_label_renderable()
        self.field_renderable = field_renderable or self.get_default_field_renderable()
        self.help_text_renderable = help_text_renderable or self.get_default_help_text_renderable()
        self.messages_container = messages_container or self.get_default_messages_container()
        super(FieldWrapper, self).__init__(**kwargs)
        self.properties['field_wrapper_renderable'] = self

    def get_default_dom_id(self):
        return '{}_wrapper'.format(self.field_renderable.dom_id)

    def bootstrap(self, **kwargs):
        """
        Extends :meth:`django_cradmin.uicontainer.container.container.AbstractContainerRenderable.bootstrap`
        with a call to
        :meth:`django_cradmin.uicontainer.form.Form.register_field_wrapper_renderable`
        to register this with the form.
        """
        returnvalue = super(FieldWrapper, self).bootstrap(**kwargs)
        self.formrenderable.register_field_wrapper_renderable(field_wrapper_renderable=self)
        return returnvalue

    def get_default_html_tag(self):
        return 'div'

    def get_default_label_renderable(self):
        """
        Get the default label renderable.

        This is used unless it is overridden using the ``label_renderable``
        kwarg for :meth:`.__init__`.

        Defaults to an object of :class:`~django_cradmin.uicontainer.label.Label`.
        """
        return label.Label()

    def get_default_field_renderable(self):
        """
        Get the default field renderable.

        This is used unless it is overridden using the ``field_renderable``
        kwarg for :meth:`.__init__`.

        Defaults to an object of
        :class:`~django_cradmin.uicontainer.field.Field`.
        """
        return field.Field()

    def get_default_help_text_renderable(self):
        """
        Get the default help text renderable.

        This is used unless it is overridden using the ``help_text_renderable``
        kwarg for :meth:`.__init__`.

        Defaults to an object of :class:`~django_cradmin.uicontainer.help_text.AutomaticHelpText`.
        """
        return field_help_text.AutomaticHelpText()

    def get_default_messages_container(self):
        """
        Get the default message container.

        This is used unless it is overridden using the ``messages_container``
        kwarg for :meth:`.__init__`.

        Defaults to a :class:`django_cradmin.uicontainer.messagecontainer.MessagesContainer`.

        Must implement :class:`django_cradmin.uicontainer.messagecontainer.AbstractMessageListMixin`.
        """
        return messagescontainer.MessagesContainer(
            test_css_class_suffixes_list=['field-messages']
        )

    def prepopulate_virtual_children_list(self):
        """
        Pre-populates the virtual children list with:

        - Label renderable
        - Field renderable
        """
        return [
            self.label_renderable,
            self.field_renderable
        ]

    def prepopulate_children_list(self):
        """
        Pre-populates the children list with:

        - Help text renderable.
        - Message renderable.

        See :meth:`django_cradmin.uicontainer.container.container.AbstractContainerRenderable.prepopulate_children_list`
        for details about what this method works, and what it should return.
        """
        return [
            self.help_text_renderable,
            self.messages_container,
        ]

    @property
    def bound_formfield(self):
        """
        Get the :class:`django.forms.boundfield.BoundField` for the form field.
        """
        return self.formrenderable.form[self.fieldname]

    # def get_default_css_classes_list(self):
    #     css_classes = super(FieldWrapper, self).get_default_css_classes_list()
    #     css_classes.append('field')
    #     return css_classes


# class NoWrapperElementFieldWrapper(FieldWrapper):
#     """
#     Same as :class:`.FieldWrapper` except that it does not include the wrapper
#     element.
#     """
#     template_name = 'django_cradmin/uicontainer/fieldwrapper_no_wrapper_element.django.html'



import django_cradmin.uicontainer.semantic
import htmls
from django import test
from django_cradmin import uicontainer


class MinimalContainer(uicontainer.container.AbstractContainerRenderable):
    pass


class TestAbstractContainerRenderable(test.TestCase):
    def test_children_as_kwarg(self):
        container = MinimalContainer(children=[MinimalContainer(), MinimalContainer()])
        self.assertEqual(container.get_childcount(), 2)

    def test_children_with_add_child(self):
        container = MinimalContainer()\
            .add_child(MinimalContainer())\
            .add_child(MinimalContainer())
        self.assertEqual(container.get_childcount(), 2)

    def test_children_with_add_children(self):
        container = MinimalContainer()\
            .add_children(MinimalContainer(), MinimalContainer())
        self.assertEqual(container.get_childcount(), 2)

    def test_dom_id_default(self):
        container = MinimalContainer()
        self.assertEqual(container.dom_id, False)

    def test_dom_id_default_overridden(self):
        class MyContainer(MinimalContainer):
            def get_default_dom_id(self):
                return 'a'
        container = MyContainer()
        self.assertEqual(container.dom_id, 'a')

    def test_dom_id_kwarg(self):
        container = MinimalContainer(dom_id='id_a')
        self.assertEqual(container.dom_id, 'id_a')

    def test_dom_id_kwarg_invalid(self):
        with self.assertRaises(uicontainer.container.InvalidDomIdError):
            MinimalContainer(dom_id='test')

    def test_dom_id_kwarg_invalid_validation_disabled(self):
        with self.settings(DJANGO_CRADMIN_UICONTAINER_VALIDATE_DOM_ID=False):
            MinimalContainer(dom_id='test')  # No InvalidDomIdError

    def test_role_default(self):
        container = MinimalContainer()
        self.assertEqual(container.role, False)

    def test_role_default_overridden(self):
        class MyContainer(MinimalContainer):
            def get_default_role(self):
                return 'a'
        container = MyContainer()
        self.assertEqual(container.role, 'a')

    def test_role_kwarg(self):
        container = MinimalContainer(role='a')
        self.assertEqual(container.role, 'a')

    def test_get_css_classes_list_default(self):
        container = MinimalContainer(role='a')
        self.assertEqual(container.get_css_classes_list(), [])

    def test_get_css_classes_list_override_default(self):
        class MyContainer(MinimalContainer):
            def get_default_css_classes_list(self):
                return ['a', 'b']
        container = MyContainer()
        self.assertEqual(container.get_css_classes_list(), ['a', 'b'])

    def test_get_css_classes_list_css_classes_list_kwarg(self):
        class MyContainer(MinimalContainer):
            def get_default_css_classes_list(self):
                return ['a', 'b']
        container = MyContainer(css_classes_list=['c', 'd'])
        self.assertEqual(container.get_css_classes_list(), ['c', 'd'])

    def test_get_css_classes_list_extra_css_classes_list_kwarg(self):
        class MyContainer(MinimalContainer):
            def get_default_css_classes_list(self):
                return ['a', 'b']
        container = MyContainer(extra_css_classes_list=['c', 'd'])
        self.assertEqual(container.get_css_classes_list(), ['a', 'b', 'c', 'd'])

    def test_get_css_classes_list_css_classes_list_and_extra_css_classes_list_kwarg(self):
        class MyContainer(MinimalContainer):
            def get_default_css_classes_list(self):
                return ['a', 'b']
        container = MyContainer(css_classes_list=['x'], extra_css_classes_list=['c', 'd'])
        self.assertEqual(container.get_css_classes_list(), ['x', 'c', 'd'])

    def test_get_bem_block_kwarg_invalid(self):
        with self.assertRaises(uicontainer.container.InvalidBemError):
            MinimalContainer(bem_block='menu__item')

    def test_get_bem_block_kwarg_invalid_validation_disabled(self):
        with self.settings(DJANGO_CRADMIN_UICONTAINER_VALIDATE_BEM=False):
            MinimalContainer(bem_block='menu__item')  # No InvalidBemError

    def test_get_css_classes_list_bem_block_kwarg(self):
        container = MinimalContainer(bem_block='menu')
        self.assertEqual(container.get_css_classes_list(), ['menu'])

    def test_get_bem_element_kwarg_invalid(self):
        with self.assertRaises(uicontainer.container.InvalidBemError):
            MinimalContainer(bem_element='menu')

    def test_get_bem_element_kwarg_invalid_validation_disabled(self):
        with self.settings(DJANGO_CRADMIN_UICONTAINER_VALIDATE_BEM=False):
            MinimalContainer(bem_element='menu')  # No InvalidBemError

    def test_get_css_classes_list_bem_element_kwarg(self):
        container = MinimalContainer(bem_element='menu__item')
        self.assertEqual(container.get_css_classes_list(), ['menu__item'])

    def test_get_css_classes_list_bem_variants_kwarg(self):
        container = MinimalContainer(bem_element='menu__item', bem_variant_list=['active'])
        self.assertEqual(container.get_css_classes_list(), ['menu__item', 'menu__item--active'])

    def test_get_css_classes_list_bem_multiple_variants(self):
        container = MinimalContainer(bem_element='menu__item', bem_variant_list=['active', 'large'])
        self.assertEqual(container.get_css_classes_list(),
                         ['menu__item', 'menu__item--active', 'menu__item--large'])

    def test_get_css_classes_list_get_default_bem_block_or_element(self):
        class MyContainer(MinimalContainer):
            def get_default_bem_block_or_element(self):
                return 'menu'
        container = MyContainer()
        self.assertEqual(container.get_css_classes_list(), ['menu'])

    def test_get_css_classes_list_get_default_bem_variant_list(self):
        class MyContainer(MinimalContainer):
            def get_default_bem_variant_list(self):
                return ['expanded']
        container = MyContainer(bem_block='menu')
        self.assertEqual(container.get_css_classes_list(), ['menu', 'menu--expanded'])

    def test_get_css_classes_list_bem_does_not_affect_other_css_classes(self):
        container = MinimalContainer(bem_block='menu',
                                     bem_variant_list=['expanded'],
                                     css_classes_list=['someclass'])
        self.assertEqual(container.get_css_classes_list(), ['menu', 'menu--expanded', 'someclass'])

    def test_iter_children(self):
        container = MinimalContainer(children=[
            MinimalContainer(dom_id='id_a'), MinimalContainer(dom_id='id_b')])
        children = list(container.iter_children())
        self.assertEqual(children[0].dom_id, 'id_a')
        self.assertEqual(children[1].dom_id, 'id_b')

    def test_get_html_element_attributes_default(self):
        container = MinimalContainer()
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            self.assertEqual(container.get_html_element_attributes(), {
                'role': False,
                'id': False,
                'class': False,
            })

    def test_get_html_element_attributes_role(self):
        container = MinimalContainer(role='a')
        self.assertEqual(container.get_html_element_attributes()['role'], 'a')

    def test_get_html_element_attributes_dom_id(self):
        container = MinimalContainer(dom_id='id_a')
        self.assertEqual(container.get_html_element_attributes()['id'], 'id_a')

    def test_get_html_element_attributes_dom_css_classes(self):
        container = MinimalContainer(css_classes_list=['a', 'b'])
        self.assertEqual(set(container.get_html_element_attributes()['class'].split()),
                         {'a', 'b', 'test-uicontainer-minimalcontainer'})

    def test_html_element_attributes_string_default(self):
        container = MinimalContainer()
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            self.assertEqual(container.html_element_attributes_string, '')

    def test_html_element_attributes_string_role(self):
        container = MinimalContainer(role="a")
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            self.assertEqual(container.html_element_attributes_string, ' role="a"')

    def test_html_element_attributes_string_dom_id(self):
        container = MinimalContainer(dom_id="id_a")
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            self.assertEqual(container.html_element_attributes_string, ' id="id_a"')

    def test_html_element_attributes_string_css_classes(self):
        container = MinimalContainer(css_classes_list=['a'])
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            self.assertEqual(container.html_element_attributes_string, ' class="a"')

    def test_html_element_attributes_string_html_element_attributes_kwarg(self):
        container = MinimalContainer(html_element_attributes={'a': '10'})
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            self.assertEqual(container.html_element_attributes_string, ' a="10"')

    def test_prepopulate_children_list_default(self):
        container = MinimalContainer()
        self.assertEqual(container.get_childcount(), 0)

    def test_prepopulate_children_list_overridden_no_other_children_added(self):
        the_child = MinimalContainer()

        class MyContainer(MinimalContainer):
            def prepopulate_children_list(self):
                return [the_child]
        container = MyContainer()
        self.assertEqual(container.get_childcount(), 1)
        self.assertEqual(list(container.iter_children())[0], the_child)

    def test_prepopulate_children_list_overridden_other_children_added(self):
        class MyContainer(MinimalContainer):
            def prepopulate_children_list(self):
                return [MinimalContainer(dom_id='id_a')]
        container = MyContainer(children=[MinimalContainer(dom_id='id_b')])
        children = list(container.iter_children())
        self.assertEqual(children[0].dom_id, 'id_a')
        self.assertEqual(children[1].dom_id, 'id_b')

    def test_prepopulate_virtual_children_list_default(self):
        container = MinimalContainer()
        self.assertEqual(container.get_virtual_childcount(), 0)

    def test_prepopulate_virtual_children_list_overridden_no_other_children_added(self):
        the_child = MinimalContainer()

        class MyContainer(MinimalContainer):
            def prepopulate_virtual_children_list(self):
                return [the_child]
        container = MyContainer()
        self.assertEqual(container.get_virtual_childcount(), 1)
        self.assertEqual(list(container.iter_virtual_children())[0], the_child)

    def test_wrapper_element_can_have_children(self):
        container = MinimalContainer()
        self.assertTrue(container.can_have_children)

    def test_wrapper_element_can_have_children_false_can_not_add_children(self):
        class MyContainer(MinimalContainer):
            @property
            def can_have_children(self):
                return False
        container = MyContainer()
        with self.assertRaises(uicontainer.container.NotAllowedToAddChildrenError):
            container.add_child(MinimalContainer())

    def test_wrapper_element_can_have_children_uses_html_tag_supports_children(self):
        self.assertTrue(MinimalContainer().can_have_children)

        class OverriddenContainer(MinimalContainer):
            @property
            def html_tag_supports_children(self):
                return False

        self.assertFalse(OverriddenContainer().can_have_children)

    def test_bootstrap(self):
        container = MinimalContainer()
        self.assertFalse(hasattr(container, 'parent'))
        self.assertFalse(container._is_bootstrapped)
        container.bootstrap()
        self.assertTrue(hasattr(container, 'parent'))
        self.assertTrue(container._is_bootstrapped)

    def test_bootstrap_with_parent(self):
        container = MinimalContainer()
        child = MinimalContainer()
        container.add_child(child)
        container.bootstrap()
        self.assertEqual(child.parent, container)

    def test_bootstrap_already_bootstrapped(self):
        container = MinimalContainer()
        container.bootstrap()
        with self.assertRaises(uicontainer.container.AlreadyBootsrappedError):
            container.bootstrap()

    def test_bootstrap_already_bootstrapped_deep(self):
        container = MinimalContainer()
        child = MinimalContainer()
        container.add_child(child)
        child.bootstrap()
        with self.assertRaises(uicontainer.container.AlreadyBootsrappedError):
            container.bootstrap()

    def test_render_sanity(self):
        container = MinimalContainer().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.count('div'), 1)

    def test_render_dom_id(self):
        container = MinimalContainer(dom_id='id_mycontainer').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('div')['id'], 'id_mycontainer')

    def test_render_role(self):
        container = MinimalContainer(role='main').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('div')['role'], 'main')

    def test_render_cssclass(self):
        container = MinimalContainer(css_classes_list=['a']).bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('div').cssclasses_set,
                         {'a', 'test-uicontainer-minimalcontainer'})

    def test_render_html_element_attributes(self):
        container = MinimalContainer(html_element_attributes={'my-attribute': 'test'}).bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('div')['my-attribute'], 'test')

    def test_render_with_children(self):
        container = MinimalContainer(dom_id='id_main', children=[
            MinimalContainer(dom_id='id_child1'),
            MinimalContainer(dom_id='id_child2'),
        ]).bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.count('div'), 3)
        self.assertTrue(selector.exists('div#id_main'))
        self.assertTrue(selector.exists('div#id_main > div#id_child1'))
        self.assertTrue(selector.exists('div#id_main > div#id_child2'))

    def test_render_with_children_deep(self):
        container = MinimalContainer(dom_id='id_main', children=[
            MinimalContainer(dom_id='id_child1', children=[
                MinimalContainer(dom_id='id_child1_1'),
            ]),
            MinimalContainer(dom_id='id_child2'),
        ]).bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.count('div'), 4)
        self.assertTrue(selector.exists('div#id_main'))
        self.assertTrue(selector.exists('div#id_main > div#id_child1'))
        self.assertTrue(selector.exists('div#id_main > div#id_child1 > div#id_child1_1'))
        self.assertTrue(selector.exists('div#id_main > div#id_child2'))

    def test_should_render_false(self):
        class MyContainer(MinimalContainer):
            @property
            def should_render(self):
                return False
        container = MyContainer().bootstrap()
        self.assertEqual(container.render(), '')


class TestDivRenderable(test.TestCase):
    def test_sanity(self):
        container = uicontainer.container.Div().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('div'))


class TestSectionRenderable(test.TestCase):
    def test_sanity(self):
        container = django_cradmin.uicontainer.semantic.Section().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('section'))


class TestHeaderRenderable(test.TestCase):
    def test_sanity(self):
        container = django_cradmin.uicontainer.semantic.Header().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('header'))


class TestMainRenderable(test.TestCase):
    def test_sanity(self):
        container = django_cradmin.uicontainer.semantic.Main().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('main'))

    def test_default_role_is_main(self):
        container = django_cradmin.uicontainer.semantic.Main().bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('main')['role'], 'main')



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy
from django_cradmin import javascriptregistry


class CradminAppConfig(AppConfig):
    name = 'django_cradmin'
    verbose_name = ugettext_lazy("Django CRadmin")

    def ready(self):
        javascriptregistry.Registry.get_instance().add(javascriptregistry.component.CradminJavascript)


class CradminWithStyleguideAppConfig(CradminAppConfig):

    def ready(self):
        from django_cradmin.apps.cradmin_kss_styleguide import styleguide_registry
        super(CradminWithStyleguideAppConfig, self).ready()

        styleguide = styleguide_registry.CradminStyleGuide(
            unique_id='basetheme',
            label='Django CRadmin basetheme',
            appname='django_cradmin',
            sourcefolder='styles/basetheme',
            sourcefile='styleguide.scss',
            # filename_patterns=[
            #     '*cradmin_theme_base/*',
            #     '*cradmin_theme_full/1__components/*',
            #     '*cradmin_theme_full/2__components/*',
            #     '*cradmin_theme_full/3__components/*',
            #     '*cradmin_theme_full/4__components/_adminui-page-header.scss',
            # ]
        )
        styleguide_registry.Registry.get_instance().add(styleguide)

        # styleguide = styleguide_registry.CradminStyleGuide(
        #     unique_id='django_cradmin_theme_example',
        #     label='Django CRadmin example theme',
        #     appname='django_cradmin',
        #     sourcefolder='styles/cradmin_theme_example',
        #     sourcefile='styleguide.scss',
        # )
        # styleguide_registry.Registry.get_instance().add(styleguide)



from __future__ import unicode_literals
from django.views.generic.detail import DetailView as DjangoDetailView
from django_cradmin.viewhelpers.mixins import QuerysetForRoleMixin
from django_cradmin import javascriptregistry


class DetailView(javascriptregistry.viewmixin.WithinRoleViewMixin,
                 QuerysetForRoleMixin, DjangoDetailView):
    """
    Base detail view for Django cradmin views.
    """

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        return context


class DetailRoleView(DetailView):
    """
    Extends :class:`.DetailView` to streamline creating a detail view
    for the current role object.

    Just like :class:`.DetailView`, but with the get_object and
    get_queryset_for_role methods implemented to edit the current role
    object.
    """
    def get_object(self, queryset=None):
        return self.get_queryset_for_role().get()

    def get_queryset_for_role(self):
        return self.model.objects.filter(pk=self.request.cradmin_role.pk)



from __future__ import unicode_literals

from django import forms
from django.conf import settings
from django.contrib import messages
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _, ugettext_lazy
from django_cradmin import uicontainer
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata, \
    GenericTokenExpiredError
from django_cradmin.viewhelpers import formview


class RepeatPasswordForm(forms.Form):
    password1 = forms.CharField(
        label=_('Type your new password'),
        widget=forms.PasswordInput)
    password2 = forms.CharField(
        label=_('Type your new password one more time'),
        widget=forms.PasswordInput)

    def clean(self):
        cleaned_data = super(RepeatPasswordForm, self).clean()
        password1 = cleaned_data.get("password1")
        password2 = cleaned_data.get("password2")

        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError(_('The passwords do not match'))


class ResetPasswordView(formview.StandaloneFormView):
    template_name = 'cradmin_resetpassword/reset.django.html'
    form_class = RepeatPasswordForm

    def get_form_renderable(self):
        formchildren = [
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='password1',
                field_renderable=uicontainer.field.Field(
                    autofocus=True,
                )
            ),
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='password2',
            ),
            uicontainer.button.SubmitPrimary(
                text=ugettext_lazy('Reset password')),
        ]
        return uicontainer.layout.AdminuiPageSectionTight(
            children=[
                uicontainer.form.Form(
                    form=self.get_form(),
                    children=formchildren,
                    dom_id='id_django_cradmin_resetpassword_reset_form'
                )
            ]
        ).bootstrap()

    def get_context_data(self, **kwargs):
        context = super(ResetPasswordView, self).get_context_data(**kwargs)
        try:
            context['generic_token_with_metadata'] = GenericTokenWithMetadata.objects.get_and_validate(
                token=self.kwargs['token'], app='cradmin_resetpassword')
        except GenericTokenWithMetadata.DoesNotExist:
            context['generic_token_with_metadata'] = None
        except GenericTokenExpiredError:
            context['generic_token_with_metadata_is_expired'] = True
        return context

    def get_success_url(self):
        return getattr(settings, 'DJANGO_CRADMIN_RESETPASSWORD_FINISHED_REDIRECT_URL', settings.LOGIN_URL)

    def __get_success_message(self):
        return render_to_string('cradmin_resetpassword/messages/successmessage.django.html').strip()

    def form_valid(self, form):
        try:
            token = GenericTokenWithMetadata.objects.pop(
                token=self.kwargs['token'], app='cradmin_resetpassword')
        except GenericTokenWithMetadata.DoesNotExist:
            return self.render_to_response(self.get_context_data())
        else:
            raw_password = form.cleaned_data['password1']
            user = token.content_object
            user.set_password(raw_password)
            user.save()
            messages.success(self.request, self.__get_success_message())
            return super(ResetPasswordView, self).form_valid(form)



import htmls
from django import test
from django_cradmin import uicontainer


class TestBlocklistItemTitle(test.TestCase):
    def test_sanity(self):
        container = uicontainer.blocklist.BlocklistItemTitle(html_tag='h1').bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('h1'))

    def test_default_css_classes(self):
        container = uicontainer.blocklist.BlocklistItemTitle(html_tag='h1').bootstrap()
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            selector = htmls.S(container.render())
        self.assertEqual(selector.one('h1')['class'], 'blocklist__itemtitle')

    def test_no_text(self):
        container = uicontainer.blocklist.BlocklistItemTitle(html_tag='h1').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h1').alltext_normalized, '')

    def test_has_text(self):
        container = uicontainer.blocklist.BlocklistItemTitle(html_tag='h1', text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('h1').alltext_normalized, 'test')


class TestBlocklistItem(test.TestCase):
    def test_sanity(self):
        container = uicontainer.blocklist.BlocklistItem().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('div'))

    def test_default_css_classes(self):
        container = uicontainer.blocklist.BlocklistItem().bootstrap()
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            selector = htmls.S(container.render())
        self.assertEqual(selector.one('div')['class'], 'blocklist__item')

    def test_no_text(self):
        container = uicontainer.blocklist.BlocklistItem().bootstrap()
        selector = htmls.S(container.render())
        self.assertFalse(selector.exists('div p'))

    def test_has_text(self):
        container = uicontainer.blocklist.BlocklistItem(text='test').bootstrap()
        selector = htmls.S(container.render())
        self.assertEqual(selector.one('div p').alltext_normalized, 'test')


class TestBlocklist(test.TestCase):
    def test_sanity(self):
        container = uicontainer.blocklist.Blocklist().bootstrap()
        selector = htmls.S(container.render())
        self.assertTrue(selector.exists('div'))

    def test_default_css_classes(self):
        container = uicontainer.blocklist.Blocklist().bootstrap()
        with self.settings(DJANGO_CRADMIN_INCLUDE_TEST_CSS_CLASSES=False):
            selector = htmls.S(container.render())
        self.assertEqual(selector.one('div')['class'], 'blocklist')

    def test_with_children(self):
        container = uicontainer.blocklist.Blocklist(
            children=[
                uicontainer.blocklist.BlocklistItem(text='a'),
                uicontainer.blocklist.BlocklistItem(text='b'),
            ]
        ).bootstrap()
        selector = htmls.S(container.render())
        child_text = [child.alltext_normalized for child in selector.list('.blocklist__item')]
        self.assertEqual(child_text[0], 'a')
        self.assertEqual(child_text[1], 'b')



from __future__ import unicode_literals

import json
from xml.sax.saxutils import quoteattr

from django_cradmin.viewhelpers.listbuilder import base
from builtins import str


class FocusBox(base.ItemValueRenderer):
    """
    Renders a value item as a box styled with the cradmin focus bg as background.

    This is a good default when setting up the structure of views,
    but you will most likely want to create your own
    :class:`django_cradmin.viewhelpers.listbuilder.base.ItemValueRenderer`
    subclass for more than the most simple use cases.
    """
    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(FocusBox, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-focus-box')
        return css_class_suffixes

    def get_base_css_classes_list(self):
        css_classes = super(FocusBox, self).get_base_css_classes_list()
        css_classes.append('django-cradmin-listbuilder-itemvalue-focusbox')
        return css_classes


class TitleDescription(FocusBox):
    """
    Extends :class:`.FocusBox` with a template and methods
    that makes it easy to render a view with a title and
    an optional description.
    """

    #: The template used to render this itemvalue.
    #: The template has lots of blocks that you can override.
    template_name = 'django_cradmin/viewhelpers/listbuilder/itemvalue/titledescription.django.html'

    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(FocusBox, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-title-description')
        return css_class_suffixes

    def get_base_css_classes_list(self):
        """
        Adds the ``django-cradmin-listbuilder-itemvalue-titledescription`` css class
        in addition to the classes added by the superclasses.
        """
        css_classes = super(TitleDescription, self).get_base_css_classes_list()
        css_classes.append('django-cradmin-listbuilder-itemvalue-titledescription')
        return css_classes

    def get_title(self):
        """
        Get the title of the box.

        Defaults to ``str(self.value)``.
        """
        return str(self.value)

    def get_description(self):
        """
        Get the description (shown below the title).

        Defaults to ``None``, which means that no description
        is rendered.
        """
        return None


class UseThis(TitleDescription):
    """
    Renders a value item with a *Use this* button that uses
    the ``django-cradmin-use-this`` directive.
    """

    #: The template used to render this itemvalue.
    #: The template has lots of blocks that you can override.
    template_name = 'django_cradmin/viewhelpers/listbuilder/itemvalue/use-this.django.html'

    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(FocusBox, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-use-this')
        return css_class_suffixes

    def get_base_css_classes_list(self):
        """
        Adds the ``django-cradmin-listbuilder-itemvalue-usethis`` css class
        in addition to the classes added by the superclasses.
        """
        css_classes = super(UseThis, self).get_base_css_classes_list()
        css_classes.append('django-cradmin-listbuilder-itemvalue-usethis')
        return css_classes

    def get_use_this_directive_options_dict(self, request):
        return {
            'value': self.value.pk,
            'fieldid': request.GET['foreignkey_select_fieldid'],
            'preview': self.get_title()
        }

    def get_use_this_directive_options_json(self, request):
        return quoteattr(json.dumps(self.get_use_this_directive_options_dict(request=request)))

    def get_context_data(self, request=None):
        context = super(UseThis, self).get_context_data(request=request)
        context['use_this_directive_options_json'] = self.get_use_this_directive_options_json(request=request)
        return context


class EditDelete(TitleDescription):
    """
    Extends :class:`.TitleDescription` with a template that makes it very easy
    to render a box that provides edit and delete buttons.

    The renderer also allows you to specify a description,
    and the template has lots of blocks that makes it easy to insert
    more content or override the existing content.
    """

    #: The template used to render this itemvalue.
    #: The template has lots of blocks that you can override.
    template_name = 'django_cradmin/viewhelpers/listbuilder/itemvalue/edit-delete.django.html'

    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(FocusBox, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-edit-delete')
        return css_class_suffixes

    def get_base_css_classes_list(self):
        """
        Adds the ``django-cradmin-listbuilder-itemvalue-titleeditdelete`` css class
        in addition to the classes added by the superclasses.
        """
        css_classes = super(EditDelete, self).get_base_css_classes_list()
        css_classes.append('django-cradmin-listbuilder-itemvalue-editdelete')
        return css_classes

    def get_edit_viewname(self):
        """
        Get the viewname within the current :class:`django_cradmin.crapp.App`
        to go to when editing. The view is called with ``self.value.id`` as
        argument. If you want to change this behavior, override the
        ``editbutton-url`` template block.

        This returns ``"edit"`` by default, and we recommend that you
        name the view for editing items this to keep things uniformly
        structured.
        """
        return 'edit'

    def get_delete_viewname(self):
        """
        Get the viewname within the current :class:`django_cradmin.crapp.App`
        to go to when deleting. The view is called with ``self.value.id`` as
        argument. If you want to change this behavior, override the
        ``deletebutton-url`` template block.

        This returns ``"delete"`` by default, and we recommend that you
        name the view for deleting items this to keep things uniformly
        structured.
        """
        return 'delete'

    def get_preview_viewname(self):
        """
        Get the viewname within the current :class:`django_cradmin.crapp.App`
        to go to when viewing. The view is called with ``self.value.id`` as
        argument. If you want to change this behavior, override the
        ``viewbutton-url`` template block.

        This returns ``None`` by default, which means that the button
        is not shown.

        See :class:`.EditDeleteWithPreview`.
        """
        return None


class EditDeleteWithPreviewMixin(object):
    """
    Mixin class for :class:`.EditDelete` that adds a view button that
    shows a preview via the view named ``"preview"`` in the current app
    (the viewname can be overridden in
    :meth:`.EditDeleteWithPreviewMixin.get_preview_viewname`).

    Examples:

        Add a view button to :class:`.EditDelete`::

            class EditDeleteWithPreview(EditDeleteWithPreviewMixin, EditDelete):
                pass

        .. note::

            The example above is the same as using :class:`.EditDeleteWithPreviewMixin`.
    """
    def get_preview_viewname(self):
        """
        Overrides :meth:`.EditDelete.get_preview_viewname` to make it return
        ``"preview"`` by default instead of ``None``.
        """
        return 'preview'


class EditDeleteWithPreview(EditDeleteWithPreviewMixin, EditDelete):
    """
    Shortcut for subclassing :class:`EditDeleteWithPreviewMixin`
    and :class:`.EditDelete`.
    """

    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(EditDeleteWithPreview, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-edit-delete-with-preview')
        return css_class_suffixes


class EditDeleteWithArchiveImage(EditDelete):
    """
    Extends :class:`.EditDelete` to include an image from cradmin imagearchive.

    Uses a template that extends the template in :class:`.EditDelete`.
    """

    #: The template used to render this itemvalue.
    #: The template has lots of blocks that you can override.
    template_name = 'django_cradmin/viewhelpers/listbuilder/itemvalue/edit-delete-with-archive-image.django.html'

    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(EditDeleteWithArchiveImage, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-edit-delete-with-archive-image')
        return css_class_suffixes

    def get_archiveimage(self):
        """
        Get a :class:`django_cradmin.apps.cradmin_imagearchive.models.ArchiveImage`
        object to use as the image.
        """
        raise NotImplementedError()

    def get_imagetype(self):
        """
        Get the ``imagetype`` to use to scale/format the image.

        See :func:`django_cradmin.templatetags.cradmin_image_tags.cradmin_create_archiveimage_tag`
        for more information on ``imagetype``.

        Defaults to ``"listbuilder-itemvalue-editdelete"``.
        """
        return "listbuilder-itemvalue-editdelete"

    def get_image_fallbackoptions(self):
        """
        Get the ``fallbackoptions`` to use to scale/format the image.
        These options are used when the imagetype returned by
        :meth:`.get_imagetype` is not defined in the
        :setting:`DJANGO_CRADMIN_IMAGEUTILS_IMAGETYPE_MAP` setting.

        See :func:`django_cradmin.templatetags.cradmin_image_tags.cradmin_create_archiveimage_tag`
        for more information on ``fallbackoptions``.

        Defaults to::

            {
                'width': 330,
                'height': 400,
                'crop': 'limit',
                'quality': 70,
            }
        """
        return {
            'width': 330,
            'height': 400,
            'crop': 'limit',
            'quality': 70,
        }


class EditDeleteWithArchiveImageAndPreview(EditDeleteWithPreviewMixin, EditDeleteWithArchiveImage):
    """
    Shortcut for subclassing :class:`EditDeleteWithPreviewMixin`
    and :class:`.EditDeleteWithArchiveImage`.
    """

    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(EditDeleteWithArchiveImage, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-edit-delete-with-archive-image-and-preview')
        return css_class_suffixes



from django.views.generic import TemplateView
from django_cradmin import javascriptregistry

from django_cradmin.apps.cradmin_kss_styleguide import styleguide_registry


class ExampleView(TemplateView, javascriptregistry.viewmixin.MinimalViewMixin):
    def get_template_names(self):
        return [
            self.styleguideconfig.get_example_template_name()
        ]

    def get_styleguideconfig(self):
        styleguideconfig = styleguide_registry.Registry.get_instance()[self.kwargs['unique_id']]
        return styleguideconfig

    def dispatch(self, request, *args, **kwargs):
        self.styleguideconfig = self.get_styleguideconfig()
        return super(ExampleView, self).dispatch(request, *args, **kwargs)

    def get_javascriptregistry_component_ids(self):
        return self.styleguideconfig.javascript_component_ids

    def get_context_data(self, **kwargs):
        context = super(ExampleView, self).get_context_data(**kwargs)
        self.styleguideconfig = self.get_styleguideconfig()
        kss_styleguide = self.styleguideconfig.make_kss_styleguide()
        section = kss_styleguide.get_section_by_reference(self.kwargs['section'])
        example = section.examples[int(self.kwargs['exampleindex'])]
        self.add_javascriptregistry_component_ids_to_context(context=context)
        context['styleguideconfig'] = self.styleguideconfig
        context['section'] = section
        context['exampleindex'] = self.kwargs['exampleindex']
        context['example'] = example
        return context



from __future__ import unicode_literals

from builtins import object

from django.conf import settings
from django.conf.urls import url, include
from django.core.urlresolvers import reverse
from django.shortcuts import render
from django.utils.html import format_html
from django.views import View
from django_cradmin import crheader
from django_cradmin import crmenu
from django_cradmin.decorators import has_access_to_cradmin_instance

from . import crapp
from .registry import cradmin_instance_registry
from .views import roleselect


def reverse_cradmin_url(instanceid, appname=None, roleid=None,
                        viewname=crapp.INDEXVIEW_NAME,
                        args=None, kwargs=None):
    """
    Reverse an URL within a cradmin instance.

    Usage is very similar to :func:`django.core.urlresolvers.reverse`,
    but you specify the cradmin instance, appname, roleid and viewname
    instead of the url-name

    Examples:

        Reverse the frontpage on an app::

            myapp_index_url = reverse_cradmin_url(
                instanceid='siteadmin',
                appname='myapp',
                roleid=site.id)

        Reverse a specific view within an app::

            myapp_add_url = reverse_cradmin_url(
                instanceid='siteadmin',
                appname='myapp',
                roleid=site.id,
                viewname='add')
    """
    if appname:
        if roleid:
            if args:
                args = [roleid] + list(args)
            else:
                if not kwargs:
                    kwargs = {}
                kwargs['roleid'] = roleid
        urlname = u'{}-{}-{}'.format(instanceid, appname, viewname)
    else:
        urlname = instanceid
    return reverse(urlname, args=args, kwargs=kwargs)


class FakeRoleFrontpageView(View):
    def dispatch(self, request, *args, **kwargs):
        raise Exception('This is just a fake view - it should not be possible to access. '
                        'The most common reason for this error is that you have configured '
                        'the BaseCrAdminInstance with flatten_rolefrontpage_url, and not '
                        'provided a URL at the root of the the rolefrontpage_appname '
                        'crapp.App.')


class BaseCrAdminInstance(object):
    """
    Base class for a django_cradmin instance.

    You define a subclass of this to setup a django_cradmin instance.

    Attributes:
        request (HttpRequest): The current HttpRequest.
    """
    #: The ID of the cradmin instance. Must be unique for the Django
    #: instance/site. Must be a string.
    #: This is typically a short readable slug that describes what
    #: the cradmin instance does.
    #: You do not need to specify this except you need to communicate/link
    #: between cradmin instances.
    id = None

    #: The regex for matching the role id.
    #: Defaults to ``\d+``.
    roleid_regex = r'\d+'

    #: The renderable class for the main menu.
    #: See :meth:`.get_main_menu_renderable`.
    main_menu_renderable_class = crmenu.DefaultMainMenuRenderable

    #: The renderable class for the expandable menu
    #: (the menu that is toggled by a button in the main menu).
    #: See :meth:`.get_expandable_menu_renderable`.
    expandable_menu_renderable_class = crmenu.DefaultExpandableMenuRenderable

    #: The header class for this cradmin instance.
    #: Must be a subclass of :class:`django_cradmin.crheader.AbstractHeaderRenderable`.
    header_renderable_class = crheader.DefaultHeaderRenderable

    #: The footer class for this cradmin instance.
    #: Must be a subclass of :class:`django_cradmin.crfooter.AbstractFooter`.
    footer_renderable_class = None

    #: The class defining the role for this cradmin instance.
    #: If you do not set this, the role system will not be used,
    #: which means that you will not get a value in ``request.cradmin_role``.
    roleclass = None

    #: The name of the app that the user should be redirected to
    #: after selecting a role. Subclasses MUST eighter specify
    #: this or override :meth:`rolefrontpage_url`.
    rolefrontpage_appname = None

    #: If this is ``True``, we do not prefix the urls of the
    #: :obj:`~BaseCrAdminInstance.rolefrontpage_appname` with the
    #: appname. This means that it is hosted on ``/baseurl/<roleid>/``
    #: instead of ``/baseurl/<roleid>/<appname>/``.
    #:
    #: If you couple this with setting :obj:`~.BaseCrAdminInstance.roleclass`
    #: to ``None``, the frontpage will be hosted directly on ``/baseurl/``.
    #:
    #: If you set this to ``True``, you have to be ensure that the urls of any views
    #: within the rolefrontpage app does crash any urls in any of the other apps.
    flatten_rolefrontpage_url = False

    #: Apps within the instance.
    #: Iterable of ``(appname, appclass)`` tuples where ``appname``
    #: is a slug for the app and ``appclass`` is a subclass of
    #: :class:`django_cradmin.crapp.App`.
    #: Can also be specified by overriding :meth:`.get_apps`.
    apps = []

    def __init__(self, request):
        """
        Parameters:
            request (HttpRequest): The current HttpRequest.
                Stored in :attr:`.request`.
        """
        self.request = request

    def get_cradmin_theme_path(self):
        """
        Return a path to a theme in the same format as :setting:`DJANGO_CRADMIN_THEME_PATH`,
        to use a custom theme for this instance.
        """
        return None

    def get_rolequeryset(self):
        """
        Get the roles for the authenticated user.

        You get the authenticated user from ``self.request.user``.
        """
        raise NotImplementedError()

    def get_titletext_for_role(self, role):
        """
        Get a short title briefly describing the given ``role``.
        """
        raise NotImplementedError()

    def get_descriptiontext_for_role(self, role):
        """
        Get a longer description for the given ``role``.

        This is never used directly on its own - it is just the default text
        used by :meth:`.get_descriptionhtml_for_role`. If you want
        HTML in your description, override :meth:`.get_descriptionhtml_for_role`
        instead.
        """
        return None

    def get_descriptionhtml_for_role(self, role):
        """
        Get a longer description for the given ``role``.
        This is always shown after/below :meth:`.get_titletext_for_role`.

        Defaults to :meth:`.get_descriptiontext_for_role` filtered to
        make it HTML safe and wrapped in a paragraph tag.
        """
        descriptiontext = self.get_descriptiontext_for_role(role)
        if descriptiontext:
            return format_html(u'<p>{}</p>', descriptiontext)
        else:
            return ''

    def get_roleid(self, role):
        """
        Get the ID for the given ``role``.
        """
        return role.pk

    def get_role_from_roleid(self, roleid):
        """
        Get the role for the given ``roleid``.

        Defaults to looking up a :obj:`roleclass` object where
        ``pk==roleid``.

        Returns:
            A role object or ``None``.
        """
        try:
            return self.roleclass.objects.get(pk=roleid)
        except self.roleclass.DoesNotExist:
            pass
        return None

    def invalid_roleid_response(self, roleid):
        """
        This is called whenever someone requests a role slug that does not exist
        (if :meth:.`get_role_from_roleid`) returns ``None``.

        Returns:
            django.http.HttpResponse: Defaults to rendering
                ``django_cradmin/invalid_roleid.django.html``.
        """
        return render(self.request, 'django_cradmin/invalid_roleid.django.html', {
            'roleid': roleid
        })

    def get_role_from_rolequeryset(self, role):
        """
        Returns the given role extracted via the :meth:`.get_rolequeryset`
        queryset.

        Raises ObjectDoesNotExist if the role is not found in the queryset.
        """
        return self.get_rolequeryset().get(pk=role.pk)

    def missing_role_response(self, role):
        """
        This is called whenever someone requests a role that exists but that
        they do not have (where meth:`.get_role_from_rolequeryset` raises ``DoesNotExist``).

        Returns:
            django.http.HttpResponse: Defaults to rendering
                ``django_cradmin/missing_role.django.html``
        """
        return render(self.request, 'django_cradmin/missing_role.django.html', {
            'role': role,
            'cradmin_javascriptregistry_component_ids': self.get_default_javascriptregistry_component_ids()
        })

    def get_menu_item_renderables(self):
        return []

    def get_expandable_menu_item_renderables(self):
        return self.get_menu_item_renderables()

    def get_main_menu_item_renderables(self):
        return self.get_menu_item_renderables()

    def get_main_menu_renderable(self):
        """
        Get the main menu renderable instance.

        Defaults to a instance of the class specified in
        :obj:`~.BaseCrAdminInstance.main_menu_renderable_class`.

        Returns:
            django_cradmin.crmenu.AbstractMenuRenderable: An AbstractMenuRenderable object.
        """
        menu_renderable = self.main_menu_renderable_class(cradmin_instance=self)
        menu_renderable.extend(self.get_main_menu_item_renderables())
        return menu_renderable

    @property
    def main_menu_renderable(self):
        if not hasattr(self, '_main_menu_renderable_cached'):
            self._main_menu_renderable_cached = self.get_main_menu_renderable()
        return self._main_menu_renderable_cached

    def get_expandable_menu_renderable(self):
        """
        Get the expandable menu renderable instance. This is the menu
        that is expanded by a button which by default is in the main
        menu.

        Defaults to a instance of the class specified in
        :obj:`~.BaseCrAdminInstance.expandable_menu_renderable_class`.

        Returns:
            django_cradmin.crmenu.AbstractMenuRenderable: An AbstractMenuRenderable object.
        """
        menu_renderable = self.expandable_menu_renderable_class(cradmin_instance=self)
        menu_renderable.extend(self.get_expandable_menu_item_renderables())
        return menu_renderable

    @property
    def expandable_menu_renderable(self):
        if not hasattr(self, '_expandable_menu_renderable_cached'):
            self._expandable_menu_renderable_cached = self.get_expandable_menu_renderable()
        return self._expandable_menu_renderable_cached

    def get_header_renderable(self):
        """
        Get the header renderable for this cradmin instance.

        Defaults to a instance of the class specified in
        :obj:`~.BaseCrAdminInstance.header_renderable_class`.

        Returns:
            django_cradmin.crheader.AbstractHeaderRenderable: An AbstractHeaderRenderable object.

        See Also:
            :class:`django_cradmin.crheader.AbstractHeaderRenderable`.
        """
        return self.header_renderable_class(cradmin_instance=self)

    def get_footer_renderable(self):
        """
        Get the footer renderable for this cradmin instance.

        Defaults to a instance of the class specified in
        :obj:`~.BaseCrAdminInstance.footer_renderable_class`.

        Returns:
            django_cradmin.crfooter.AbstractHeaderRenderable: An AbstractHeaderRenderable object.

        See Also:
            :class:`django_cradmin.crfooter.AbstractHeaderRenderable`.
        """
        if self.footer_renderable_class:
            return self.footer_renderable_class(cradmin_instance=self)
        else:
            return None

    def reverse_url(self, appname, viewname, args=None, kwargs=None, roleid=None):
        """
        Reverse an URL within this cradmin instance.

        The advantage over using :func:`django.core.urlresolvers.reverse`
        is that you do not need to hardcode the id of the cradmin instance,
        and that the ``roleid`` is automatically added to args or kwargs
        (depending on which one you use to pass arguments to the url).

        Args:
            appname (str): The name of the app.
            viewname (str): The name of the view within the app.
            args (list): Args for the view
            kwargs (dict): Keyword args for the view.
            roleid: The roleid.
        """
        kwargs = {
            'instanceid': self.id,
            'appname': appname,
            'viewname': viewname,
            'args': args,
            'kwargs': kwargs
        }
        if self.roleclass:
            if roleid is None:
                roleid = self.get_roleid(self.request.cradmin_role)
            kwargs['roleid'] = roleid
        return reverse_cradmin_url(**kwargs)

    def appindex_url(self, appname, args=None, kwargs=None, roleid=None):
        """
        Reverse the url of the landing page for the given app.

        The landing page is the view named :obj:`django_cradmin.crapp.INDEXVIEW_NAME`.

        This would be the same as using :meth:`.reverse_url` with ``viewname=crapp.INDEXVIEW_NAME``.

        Args:
            appname (str): The name of the app.
            args (list): Args for the view
            kwargs (dict): Keyword args for the view.
            roleid: The roleid.
        """
        return self.reverse_url(appname, viewname=crapp.INDEXVIEW_NAME, roleid=roleid,
                                args=args, kwargs=kwargs)

    def rolefrontpage_url(self, roleid=None):
        """
        Returns the URL that the user should be redirected to
        after selecting a role.
        """
        if self.roleclass:
            if roleid is None:
                roleid = self.get_roleid(self.request.cradmin_role)
        return self.appindex_url(self.rolefrontpage_appname, roleid=roleid)

    def get_instance_frontpage_url(self):
        """
        Return the URL of the instance frontpage view.

        See:
            :meth:`.get_instance_frontpage_url`.
        """
        if self.__class__.__no_role_and_flatten_rolefrontpage_url():
            return self.rolefrontpage_url()
        else:
            return reverse('{}-frontpage'.format(self.id))

    def roleselectview_url(self):
        """
        Deprecated, use :meth:`.get_instance_frontpage_url` instead.
        """
        return self.get_instance_frontpage_url()

    def get_common_http_headers(self):
        """
        Override this to set common HTTP headers for all views in the instance.

        Returns:
            A mapping object mapping HTTP header name to value. Returns
            empty dict by default.
        """
        return {}

    def has_access(self):
        """
        Check if the given user has access to this cradmin instance.

        Defaults to ``self.request.user.is_authenticated()``, but you
        can override this.
        """
        return self.request.user.is_authenticated()

    def get_foreignkeyselectview_url(self, model_class):
        """
        Get foreign key select view URL for the given model class.

        This can be used by foreign key select widgets to lookup a view
        for this model within the current instance.

        By default this returns ``None``, so you have to override this
        if you want to use it.

        Parameters:
            model_class: A ``django.db.models.Model`` subclass.
        """
        return None

    def get_manytomanyselectview_url(self, model_class):
        """
        Get many-to-many select view URL for the given model class.

        This can be used by many-to-many widgets,
        like :class:`django_cradmin.widgets.modelmultichoice.ModelMultiChoiceWidget`,
        to lookup a view for this model within the current instance.

        By default this returns ``None``, so you have to override this
        if you want to use it.

        Parameters:
            model_class: A ``django.db.models.Model`` subclass.
        """
        return None

    @classmethod
    def get_roleselect_view(cls):
        """
        The view for selecting role.

        Defaults to::

            from django_cradmin.views import roleselect
            from django.contrib.auth.decorators import login_required
            return login_required(roleselect.RoleSelectView.as_view())

        If you want to provide your own role select view, you can simply implement
        it here. Another option is to extend RoleSelectView and override the ``template_name``.

        Note:

            The name of the URL for this view is ``<cradmin instance id>-roleselect``,
            where ``<cradmin instance id>`` is :obj:`.id`. You can reverse the URL of
            this view with :meth:`.get_instance_frontpage_url`.
        """
        return has_access_to_cradmin_instance(cls.id, roleselect.RoleSelectView.as_view())

    @classmethod
    def get_instance_frontpage_view(cls):
        if cls.roleclass:
            return cls.get_roleselect_view()
        else:
            raise NotImplementedError('When you do not define a roleclass, you have '
                                      'to override get_instance_frontpage_view()')

    @classmethod
    def get_apps(cls):
        """
        See :obj:`.apps`. Defaults to returning :obj:`.apps`, but can be overridden.
        """
        return cls.apps

    @classmethod
    def get_app_url(cls, appname, appclass):
        appurlpatterns = appclass.build_urls(cls.id, appname)
        flatten = cls.rolefrontpage_appname == appname and cls.flatten_rolefrontpage_url
        if cls.roleclass:
            if flatten:
                return url(r'^(?P<roleid>{})/'.format(cls.roleid_regex),
                           include(appurlpatterns))
            else:
                return url(r'^(?P<roleid>{})/{}/'.format(cls.roleid_regex, appname),
                           include(appurlpatterns))

        else:
            if flatten:
                return url(r'^', include(appurlpatterns))
            else:
                return url(r'^{}/'.format(appname),
                           include(appurlpatterns))

    @classmethod
    def _get_app_urls(cls):
        urls = []
        for appname, appclass in cls.get_apps():
            urls.append(cls.get_app_url(appname=appname, appclass=appclass))
        return urls

    @classmethod
    def __no_role_and_flatten_rolefrontpage_url(cls):
        return cls.flatten_rolefrontpage_url and not cls.roleclass

    @classmethod
    def urls(cls):
        """
        Get the url patterns for the cradmin instance.
        """
        cradmin_instance_registry.add(cls)
        urls = cls._get_app_urls()
        if cls.__no_role_and_flatten_rolefrontpage_url():
            urls.append(url('^$', FakeRoleFrontpageView.as_view(),
                            name=cls.id))
        else:
            urls.append(url('^$', cls.get_instance_frontpage_view(),
                            name=cls.id))

        return urls

    def add_extra_instance_variables_to_request(self, request):
        """
        Override this method to add extra attributes to the request object
        for all views in this cradmin instance.

        This is called by the decorator that wraps all views within the instance.
        """

    def get_body_css_classes_list(self):
        """
        Get the css classes for the ``<body>`` element that this
        cradmin instance should add for all views as a list.

        Returns an empty list by default, but you should override this
        to add css classes to the ``<body>`` element for all views
        within this instance.
        """
        return []

    def get_body_css_classes_string(self):
        """
        Get the css classes for the ``<body>`` element that this
        cradmin instance should add for all views as a string.

        You should not override this - override :meth:`.get_body_css_classes_list`.
        This method only joins the list returned by that method to make it
        easier to use in Django templates.
        """
        return ' '.join(self.get_body_css_classes_list())

    @property
    def page_cover_bem_block(self):
        """
        Get the name of the BEM block for the page cover.

        Should be overridden if you do not want the default of ``adminui-page-cover``.

        If you need more complex behavior, you should consider:
        - Making your own templates that extend:
            - ``django_cradmin/standalone-base.django.html`` - for all the views outside the role.
            - ``django_cradmin/base.django.html`` - for all the views within the role
        - OR override (this affects all cradmin instances):
            - ``django_cradmin/standalone-base.django.html``
            - ``django_cradmin/base.django.html``
        """
        return 'adminui-page-cover'

    def get_default_javascriptregistry_component_ids(self):
        """
        Get default component IDs for all views within this cradmin instance.
        """
        return []

    def get_default_within_role_javascriptregistry_component_ids(self):
        """
        Get default component IDs for all views within a cradmin_role within this cradmin instance.

        Defaults to :meth:`.get_default_javascriptregistry_component_ids`.
        """
        return self.get_default_javascriptregistry_component_ids() + [
            'django_cradmin_javascript'
        ]


class NoRoleMixin(object):
    """
    Mixin to make a :class:`.BaseCrAdminInstance` not require a role.

    Must be mixed in before :class:`.BaseCrAdminInstance`.
    """
    flatten_rolefrontpage_url = True

    def get_titletext_for_role(self, role):
        return settings.DJANGO_CRADMIN_SITENAME


class NoLoginMixin(object):
    """
    Mixin to make a :class:`.BaseCrAdminInstance` not require login.

    Must be mixed in before :class:`.BaseCrAdminInstance`.
    """
    def has_access(self):
        """
        We give any user access to this instance, including unauthenticated users.
        """
        return True


class NoRoleNoLoginCrAdminInstance(NoRoleMixin, NoLoginMixin, BaseCrAdminInstance):
    """
    Shortcut for creating a :class:`.BaseCrAdminInstance` with the
     :class:`.NoRoleMixin` and :class:`.NoLoginMixin`.
    """



from django.shortcuts import redirect
from django_cradmin import viewhelpers

from django_cradmin.apps.cradmin_kss_styleguide import styleguide_registry


class GuideListView(viewhelpers.generic.WithinRoleTemplateView):
    template_name = 'cradmin_kss_styleguide/styleguideview/guides.django.html'

    def get(self, request, *args, **kwargs):
        self.styleguideregistry = styleguide_registry.Registry.get_instance()
        if len(self.styleguideregistry) == 1:
            styleguideconfig = self.styleguideregistry.first_guide()
            return redirect('cradmin_kss_styleguide_guide', unique_id=styleguideconfig.unique_id)
        else:
            return super(GuideListView, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(GuideListView, self).get_context_data(**kwargs)
        context['styleguideregistry'] = self.styleguideregistry
        context['prefix'] = self.kwargs.get('prefix', None)
        return context


class GuideView(viewhelpers.generic.WithinRoleTemplateView):
    template_name = 'cradmin_kss_styleguide/styleguideview/guide.django.html'

    def get_styleguideconfig(self):
        styleguideconfig = styleguide_registry.Registry.get_instance()[self.kwargs['unique_id']]
        return styleguideconfig

    def dispatch(self, request, *args, **kwargs):
        self.styleguideconfig = self.get_styleguideconfig()
        return super(GuideView, self).dispatch(request, *args, **kwargs)

    def get_template_names(self):
        return [
            self.styleguideconfig.get_template_name()
        ]

    def get_context_data(self, **kwargs):
        context = super(GuideView, self).get_context_data(**kwargs)
        styleguideconfig = self.get_styleguideconfig()
        context['styleguideconfig'] = styleguideconfig
        context['kss_styleguide'] = styleguideconfig.make_kss_styleguide()
        return context



from . import container


class EscapedText(container.AbstractContainerRenderable):
    """
    Renders provided text after escaping any HTML.

    Can not have children.
    """
    template_name = 'django_cradmin/uicontainer/text/escaped_text.django.html'

    def __init__(self, text, **kwargs):
        """
        Args:
            text: The text to render.
            **kwargs: Kwargs for :class:`~django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self.text = text
        super(EscapedText, self).__init__(**kwargs)

    @property
    def can_have_children(self):
        return False


class Html(container.AbstractContainerRenderable):
    """
    Renders provided HTML directly without any formatting or escaping.

    Can not have children.
    """
    template_name = 'django_cradmin/uicontainer/text/html.django.html'

    def __init__(self, html=None, **kwargs):
        """
        Args:
            html: The HTML to render.
            **kwargs: Kwargs for :class:`~django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self._overridden_html = html
        super(Html, self).__init__(**kwargs)

    @property
    def can_have_children(self):
        return False

    def get_default_html(self):
        """
        Get the default HTML to render if no HTML is provided
        as the ``html`` kwarg for :meth:`.__init__`.

        Defaults to empty string.
        """
        return ''

    @property
    def html(self):
        return self._overridden_html or self.get_default_html()



from django_cradmin.uicontainer.container import AbstractContainerRenderable

from . import convenience


class Paragraph(convenience.AbstractWithOptionalEscapedText):
    """
    Renders a ``<p>``.
    """
    def get_default_html_tag(self):
        return 'p'


class Link(convenience.AbstractWithOptionalEscapedText):
    """
    Renders a ``<a>``.
    """
    def __init__(self, href=False, **kwargs):
        """

        Args:
            href: The href attribute value. If this is ``False``
                (the default), no href attribute is rendered.
            **kwargs: Kwargs for :class:`django_cradmin.uicontainer.convenience.AbstractWithOptionalEscapedText`.
        """
        self.href = href
        super(Link, self).__init__(**kwargs)

    def get_default_html_tag(self):
        return 'a'

    def get_html_element_attributes(self):
        attributes = super(Link, self).get_html_element_attributes()
        attributes['href'] = self.href
        return attributes


class H1(convenience.AbstractWithOptionalEscapedText):
    """
    Renders a ``<h1>``.
    """
    def get_default_html_tag(self):
        return 'h1'


class H2(convenience.AbstractWithOptionalEscapedText):
    """
    Renders a ``<h2>``.
    """
    def get_default_html_tag(self):
        return 'h2'


class H3(convenience.AbstractWithOptionalEscapedText):
    """
    Renders a ``<h3>``.
    """
    def get_default_html_tag(self):
        return 'h3'


class H4(convenience.AbstractWithOptionalEscapedText):
    """
    Renders a ``<h4>``.
    """
    def get_default_html_tag(self):
        return 'h4'


class Section(AbstractContainerRenderable):
    """
    Renders a ``<section>``.
    """
    def get_default_html_tag(self):
        return 'section'


class Header(AbstractContainerRenderable):
    """
    Renders a ``<header>``.
    """
    def get_default_html_tag(self):
        return 'header'


class Footer(AbstractContainerRenderable):
    """
    Renders a ``<footer>``.
    """
    def get_default_html_tag(self):
        return 'footer'


class Main(AbstractContainerRenderable):
    """
    Renders a ``<main>``.
    """
    def get_default_html_tag(self):
        return 'main'

    def get_default_role(self):
        return 'main'


class Nav(AbstractContainerRenderable):
    """
    Renders a ``<nav>``.
    """
    def get_default_html_tag(self):
        return 'nav'


class Ul(AbstractContainerRenderable):
    """
    Renders a ``<ul>``.
    """
    def get_default_html_tag(self):
        return 'ul'


class Ol(AbstractContainerRenderable):
    """
    Renders a ``<ol>``.
    """
    def get_default_html_tag(self):
        return 'ol'


class Li(convenience.AbstractWithOptionalEscapedText):
    """
    Renders a ``<li>``.
    """
    def get_default_html_tag(self):
        return 'li'



class MinimalViewMixin(object):
    """
    Views that use the javascriptregistry template tags must
    use this mixin. The base templates for cradmin uses these template
    tags, so this means that this mixin is essential for most views.
    You will normally want to mix in one of the subclasses:

    - :class:`.StandaloneBaseViewMixin`
    - :class:`.WithinRoleViewMixin`

    Or extend one of the views in ``django_cradmin.viewhelpers``.

    Examples:

        Usage (same for the subclasses)::

            class MyView(TemplateView, javascriptregistry.viewmixin.MinimalViewMixin):
                def get_context_data(self, **kwargs):
                    context = super(MinimalViewMixin, self).get_context_data(**kwargs)
                    self.add_javascriptregistry_component_ids_to_context(context=context)
                    return context
    """

    def get_javascriptregistry_component_ids(self):
        return []

    def add_javascriptregistry_component_ids_to_context(self, context):
        """
        Call this to add the ``cradmin_javascriptregistry_component_ids`` template
        context variable to the provided ``context``.

        Do not override this in subclasses.

        Args:
            context (dict): A template context.

        """
        context['cradmin_javascriptregistry_component_ids'] = self.get_javascriptregistry_component_ids()


class StandaloneBaseViewMixin(MinimalViewMixin):
    """
    Use with views that use the ``django_cradmin/standalone-base.django.html``
    template.

    DO NOT use for views that use the ``django_cradmin/base.django.html`` template,
    use :class:`.ViewMixin` for that.
    """
    def get_javascriptregistry_component_ids(self):
        if getattr(self.request, 'cradmin_instance', None):
            return self.request.cradmin_instance.get_default_javascriptregistry_component_ids()
        else:
            return []


class WithinRoleViewMixin(StandaloneBaseViewMixin):
    """
    Use with views that use the ``django_cradmin/base.django.html`` template.
    """
    def get_javascriptregistry_component_ids(self):
        if getattr(self.request, 'cradmin_instance', None):
            return self.request.cradmin_instance.get_default_within_role_javascriptregistry_component_ids()
        else:
            return []



from django_cradmin.viewhelpers.listbuilder import base


class DefaultSpacingItemFrame(base.ItemFrameRenderer):
    """
    Renders a frame that adds some spacing around the items in the list.
    """
    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(DefaultSpacingItemFrame, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-default-spacing-item-frame')
        return css_class_suffixes

    def get_base_css_classes_list(self):
        css_classes = super(DefaultSpacingItemFrame, self).get_base_css_classes_list()
        css_classes.append('blocklist__item')
        return css_classes


class Link(DefaultSpacingItemFrame):
    """
    Renders a frame as a link.
    """
    template_name = 'django_cradmin/viewhelpers/listbuilder/itemframe/link.django.html'

    def get_test_css_class_suffixes_list(self):
        css_class_suffixes = super(DefaultSpacingItemFrame, self).get_test_css_class_suffixes_list()
        css_class_suffixes.append('cradmin-listbuilder-link')
        return css_class_suffixes

    def get_base_css_classes_list(self):
        css_classes = super(Link, self).get_base_css_classes_list()
        css_classes.append('blocklist__item--link')
        return css_classes

    def get_url(self):
        """
        Get the URL the link should go to.

        Defaults to ``value.get_absolute_url``, which means that this
        works out of the box with any Django model object with
        the ``get_absolute_url`` method defined.
        """
        return self.value.get_absolute_url()



from __future__ import unicode_literals
from django.contrib import admin


class SortableModelAdmin(admin.ModelAdmin):
    """
    ModelAdmin that automatically sets the ``sort_index`` of newly added items
    last in their parent. It also makes ``sort_index`` read-only by default.

    Used just like ``django.contrib.admin.ModelAdmin``.
    """

    #: If this is ``True`` , we make the ``sort_index`` field
    #: read-only. Override this to avoid this magic (typically for debugging).
    make_sort_index_readonly = False

    def save_model(self, request, obj, form, change):
        """
        Overridden to set the sortindex on save if the pk is None.
        """
        if obj.sort_index is None:
            if obj.pk is None:
                self.model.objects.set_newitem_sort_index_to_last(obj)
        super(SortableModelAdmin, self).save_model(request, obj, form, change)

    def get_readonly_fields(self, request, obj=None):
        """
        Overridden to make the sortindex readonly if :obj:`.make_sort_index_readonly`
        is ``True``.
        """
        readonly_fields = super(SortableModelAdmin, self).get_readonly_fields(request, obj=obj)
        if self.make_sort_index_readonly:
            readonly_fields = list(readonly_fields)
            if 'sort_index' not in readonly_fields:
                readonly_fields.append('sort_index')
        return readonly_fields



from django_cradmin.viewhelpers.listfilter.base.abstractfilter import AbstractFilter


class AbstractRadioFilter(AbstractFilter):
    """
    Abstract base class for any radio based singleselect filter.

    You only have to override :meth:`~AbstractRadioFilter.get_choices` and
    :meth:`~AbstractRadioFilter.filter`.
    """
    template_name = 'django_cradmin/viewhelpers/listfilter/django/single/radio/base.django.html'

    def get_choices(self):
        """
        Get choices as a list of of ``(value, label)`` pairs.
        """
        raise NotImplementedError()

    def get_choices_cached(self):
        if not hasattr(self, '_choices'):
            self._choices = self.get_choices()
        return self._choices

    def get_choicesdata(self):
        """
        Get choices data as a list of dicts.

        This uses :meth:`.get_choices`, and turns it into
        a list of dicts where each dict has:

        - ``url``: The URL to load when making a choice.
          This is generated using
          :meth:`~django_cradmin.viewhelpers.listfilter.base.AbstractFilter.build_set_values_url`
          with the value from :meth:`.get_choices` as input.
        - ``label``: The label to show (taken directly from :meth:`.get_choices`.
        - ``is_selected`` (boolean): If the current value matches the value of a choice,
          that choice will be selected. If none of them matches, the first choice is selected.
          You can change this behavior by overriding :meth:`.get_default_is_selected_index`.
        - ``dom_id``: The DOM ID of the checkbox input field.

        Returns:
            A tuple with the list of dicts described above and the selected label.
        """
        selected_value = self.get_cleaned_value()
        choicesdata = []
        for value, label in self.get_choices_cached():
            is_selected = value == selected_value
            url = self.build_set_values_url(values=[value])
            choicesdata.append({
                'url': url,
                'label': label,
                'is_selected': is_selected,
                'dom_id': '{}_{}'.format(self.get_inputfield_dom_id(), value)
            })
        return choicesdata

    def get_context_data(self, request=None):
        context = super(AbstractRadioFilter, self).get_context_data(request=None)
        context['choicesdata'] = self.get_choicesdata()
        return context

    def get_cleaned_value(self):
        cleaned_value = super(AbstractRadioFilter, self).get_cleaned_value()
        choices = set(value for value, label in self.get_choices_cached())
        if cleaned_value in choices:
            return cleaned_value
        else:
            return ''



from . import container  # noqa
from . import convenience  # noqa
from . import text  # noqa
from . import nothing  # noqa
from . import blocklist  # noqa
from . import messagescontainer  # noqa
from . import semantic  # noqa
from . import button  # noqa
from . import form_mixins  # noqa
from . import form  # noqa
from . import label  # noqa
from . import field_help_text  # noqa
from . import field  # noqa
from . import fieldwrapper  # noqa
from . import fieldset  # noqa
from . import layout  # noqa



import json

import htmls
from django import test

from django_cradmin import python2_compatibility
from django_cradmin.viewhelpers.multiselect2 import listbuilder_itemvalues
from django_cradmin.python2_compatibility import mock


class TestItemValue(test.TestCase):
    def test_title(self):
        mockvalue = mock.MagicMock()
        mockvalue.__str__.return_value = 'testvalue'
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = 'testvalue'

        selector = htmls.S(listbuilder_itemvalues.ItemValue(
            value=mockvalue).render())
        self.assertEqual(
            'testvalue',
            selector.one('.test-cradmin-listbuilder-title-description__title').alltext_normalized)

    def test_no_description(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(
            value=mockvalue).render())
        self.assertFalse(
            selector.exists('.test-cradmin-listbuilder-title-description__description'))

    def test_has_description(self):
        class MySelectedItem(listbuilder_itemvalues.ItemValue):
            def get_description(self):
                return 'test description'

        mockvalue = mock.MagicMock()
        selector = htmls.S(MySelectedItem(value=mockvalue).render())
        self.assertEqual(
            'test description',
            selector.one('.test-cradmin-listbuilder-title-description__description').alltext_normalized)

    def test_selectbutton_text(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(
            value=mockvalue).render())
        self.assertEqual(
            'Select',
            selector.one('button.test-cradmin-multiselect2-itemvalue-select-button').alltext_normalized)

    def test_selectbutton_aria_label(self):
        mockvalue = mock.MagicMock()
        mockvalue.__str__.return_value = 'testvalue'
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = 'testvalue'

        selector = htmls.S(listbuilder_itemvalues.ItemValue(
            value=mockvalue).render())
        self.assertEqual(
            'Select "testvalue"',
            selector.one('button.test-cradmin-multiselect2-itemvalue-select-button')['aria-label'])

    def test_selectbutton_id(self):
        mockvalue = mock.MagicMock()
        mockvalue.pk = 10
        selector = htmls.S(listbuilder_itemvalues.ItemValue(
            value=mockvalue).render())
        self.assertTrue(selector.exists('button#django_cradmin_multiselect2_selectbutton_10'))

    def test_angularjs_directive(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(
            value=mockvalue).render())
        directiveoptions = json.loads(selector.one(
            'button.test-cradmin-multiselect2-itemvalue-select-button')['django-cradmin-multiselect2-select'])
        self.assertEqual(
            {'custom_data': None,
             'is_selected': False,
             'item_wrapper_css_selector': 'li',
             'preview_container_css_selector': '.django-cradmin-multiselect2-itemvalue',
             'preview_css_selector': '.django-cradmin-multiselect2-selected-item',
             'target_dom_id': 'django_cradmin_multiselect2_select_target'},
            directiveoptions)

    def test_angularjs_directive_custom_target_dom_id(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(
            value=mockvalue, target_dom_id='testid').render())
        directiveoptions = json.loads(selector.one(
            'button.test-cradmin-multiselect2-itemvalue-select-button')['django-cradmin-multiselect2-select'])
        self.assertEqual(
            'testid',
            directiveoptions['target_dom_id'])

    def test_angularjs_directive_is_selected_false(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(
            value=mockvalue, is_selected=False).render())
        directiveoptions = json.loads(selector.one(
            'button.test-cradmin-multiselect2-itemvalue-select-button')['django-cradmin-multiselect2-select'])
        self.assertFalse(directiveoptions['is_selected'])

    def test_angularjs_directive_is_selected_true(self):
        mockvalue = mock.MagicMock()
        selector = htmls.S(listbuilder_itemvalues.ItemValue(
            value=mockvalue, is_selected=True).render())
        directiveoptions = json.loads(selector.one(
            'button.test-cradmin-multiselect2-itemvalue-select-button')['django-cradmin-multiselect2-select'])
        self.assertTrue(directiveoptions['is_selected'])


class TestManyToManySelect(test.TestCase):
    def test_sanity(self):
        mockvalue = mock.MagicMock()
        mockvalue.__str__.return_value = 'testvalue'
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = 'testvalue'
        mockvalue.pk = 'unused'
        selector = htmls.S(listbuilder_itemvalues.ManyToManySelect(
            value=mockvalue).render())
        self.assertEqual(
            'testvalue',
            selector.one('.test-cradmin-listbuilder-title-description__title').alltext_normalized)

    def test_angularjs_directive_custom_data_value(self):
        mockvalue = mock.MagicMock()
        mockvalue.pk = 21
        selector = htmls.S(listbuilder_itemvalues.ManyToManySelect(
            value=mockvalue, target_dom_id='testid').render())
        directiveoptions = json.loads(selector.one(
            'button.test-cradmin-multiselect2-itemvalue-select-button')['django-cradmin-multiselect2-select'])
        self.assertEqual(
            21,
            directiveoptions['custom_data']['value'])

    def test_angularjs_directive_custom_data_preview(self):
        mockvalue = mock.MagicMock()
        mockvalue.pk = 'unused'
        mockvalue.__str__.return_value = 'testvalue'
        if python2_compatibility.is_python2():
            mockvalue.__unicode__.return_value = 'testvalue'
        selector = htmls.S(listbuilder_itemvalues.ManyToManySelect(
            value=mockvalue, target_dom_id='testid').render())
        directiveoptions = json.loads(selector.one(
            'button.test-cradmin-multiselect2-itemvalue-select-button')['django-cradmin-multiselect2-select'])
        selector = htmls.S(directiveoptions['custom_data']['preview'])
        self.assertEqual(
            'testvalue',
            selector.one('.test-cradmin-multiselect2-preview-list-value').alltext_normalized)



from django import test

from django_cradmin import javascriptregistry
from django_cradmin.javascriptregistry.component import AbstractJsComponent


class MinimalAbstractJsComponent(javascriptregistry.component.AbstractJsComponent):
    @classmethod
    def get_component_id(cls):
        return 'minimal'


class TestAbstractJsComponent(test.TestCase):
    def test_get_component_id(self):
        with self.assertRaises(NotImplementedError):
            javascriptregistry.component.AbstractJsComponent.get_component_id()

    def test__init__component_id_uppercase_not_allowed(self):
        class MockJsComponent(AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'Hello'

        with self.assertRaises(javascriptregistry.component.ComponentIdFormatError):
            MockJsComponent(request=None)

    def test__init__component_id_minus_not_allowed(self):
        class MockJsComponent(AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'he-lo'

        with self.assertRaises(javascriptregistry.component.ComponentIdFormatError):
            MockJsComponent(request=None)

    def test__init__component_id_underscore_last_not_allowed(self):
        class MockJsComponent(AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'hello_'

        with self.assertRaises(javascriptregistry.component.ComponentIdFormatError):
            MockJsComponent(request=None)

    def test__init__component_id_underscore_first_not_allowed(self):
        class MockJsComponent(AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return '_hello'

        with self.assertRaises(javascriptregistry.component.ComponentIdFormatError):
            MockJsComponent(request=None)

    def test__init__component_id_number_first_not_allowed(self):
        class MockJsComponent(AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return '1hello'

        with self.assertRaises(javascriptregistry.component.ComponentIdFormatError):
            MockJsComponent(request=None)

    def test__init__component_id_underscore_middle_allowed(self):
        class MockJsComponent(AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'he_lo'

        MockJsComponent(request=None)  # No ComponentIdFormatError

    def test__init__component_id_number_middle_allowed(self):
        class MockJsComponent(AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'he3lo'

        MockJsComponent(request=None)  # No ComponentIdFormatError

    def test__init__component_id_number_last_allowed(self):
        class MockJsComponent(AbstractJsComponent):
            @classmethod
            def get_component_id(cls):
                return 'hello1'

        MockJsComponent(request=None)  # No ComponentIdFormatError

    def test_get_dependencies(self):
        self.assertEqual([], MinimalAbstractJsComponent(request=None).get_dependencies())

    def test_get_head_sourceurls(self):
        self.assertEqual([], MinimalAbstractJsComponent(request=None).get_head_sourceurls())

    def test_get_sourceurls(self):
        self.assertEqual([], MinimalAbstractJsComponent(request=None).get_sourceurls())

    def test_get_javascript_code_before_sourceurls(self):
        self.assertIsNone(MinimalAbstractJsComponent(request=None).get_javascript_code_before_sourceurls())

    def test_get_javascript_code_after_sourceurls(self):
        self.assertIsNone(MinimalAbstractJsComponent(request=None).get_javascript_code_after_sourceurls())

    def test_get_static_url(self):
        self.assertEqual(
            '/static/test', MinimalAbstractJsComponent(request=None).get_static_url(path='test'))


class MinimalCradminAngular1(javascriptregistry.component.CradminAngular1):
    @classmethod
    def get_component_id(cls):
        return 'minimal'

    def get_target_domelement_selector(self):
        return '#id_test'


class TestCradminAngular1(test.TestCase):
    def test_get_component_id(self):
        with self.assertRaises(NotImplementedError):
            javascriptregistry.component.AbstractJsComponent.get_component_id()

    def test_get_target_domelement_selector(self):
        class MockCradminAngular1(javascriptregistry.component.CradminAngular1):
            @classmethod
            def get_component_id(cls):
                return 'minimal'

        with self.assertRaises(NotImplementedError):
            MockCradminAngular1(request=None).get_target_domelement_selector()

    def test_get_head_sourceurls(self):
        self.assertEqual(
            [
                '/static/django_cradmin/dist/vendor/cradmin-vendorjs.js',
                '/static/django_cradmin/dist/js/cradmin.min.js'
            ],
            MinimalCradminAngular1(request=None).get_head_sourceurls())

    def test_get_angularjs_modules(self):
        self.assertEqual(
            [],
            MinimalCradminAngular1(request=None).get_angularjs_modules())

    def test_get_javascript_code_after_sourceurls(self):
        class MockCradminAngular1(javascriptregistry.component.CradminAngular1):
            @classmethod
            def get_component_id(cls):
                return 'mock'

            def get_target_domelement_selector(self):
                return '#id_mock'

        js = MockCradminAngular1(request=None).get_javascript_code_after_sourceurls()

        self.assertTrue(
            'angular.module("MockApp", []);\n'
            'angular.bootstrap(document.querySelector("#id_mock"), ["MockApp"])',
            js)

    def test_get_javascript_code_after_sourceurls_with_modules(self):
        class MockCradminAngular1(javascriptregistry.component.CradminAngular1):
            @classmethod
            def get_component_id(cls):
                return 'mock'

            def get_target_domelement_selector(self):
                return '#id_mock'

            def get_angularjs_modules(self):
                return ['myapp.forms', 'myapp.modal']

        js = MockCradminAngular1(request=None).get_javascript_code_after_sourceurls()

        self.assertTrue(
            'angular.module("MockApp", ["myapp.forms", "myapp.modal"]);\n'
            'angular.bootstrap(document.querySelector("#id_mock"), ["MockApp"])',
            js)


class TestCradminJavascript(test.TestCase):
    def test_get_component_id(self):
        self.assertEqual(
            'django_cradmin_javascript',
            javascriptregistry.component.CradminJavascript.get_component_id())



import json
from xml.sax.saxutils import quoteattr

from django import forms
from django.contrib import messages
from django.db import models
from django.views.generic.edit import FormMixin

from django_cradmin.viewhelpers import listbuilderview
from django_cradmin.viewhelpers import multiselect2


class ViewMixin(FormMixin):
    """
    Multiselect2 view mixin. Must be mixin in before any Django View subclass.

    This is typically used with a Django ListView or TemplateView.
    """
    def get_target_renderer_class(self):
        """
        Get the target renderer class.

        Must return :class:`django_cradmin.viewhelpers.multiselect2.target_renderer.Target`
        or a subclass of that class.

        Defaults to returning :class:`django_cradmin.viewhelpers.multiselect2.target_renderer.Target`.
        """
        return multiselect2.target_renderer.Target

    def get_target_renderer_kwargs(self):
        """
        Get keywork arguments for the class returned
        by :meth:`.get_target_renderer_class`

        You should call ``super()`` when overriding this method
        to get any default kwargs provided by this method.
        """
        return {
            'form': self.get_form(form_class=self.get_form_class())
        }

    def get_target_renderer(self):
        """
        Get the target renderer object.

        Must return an object of :class:`django_cradmin.viewhelpers.multiselect2.target_renderer.Target`
        or a subclass of that class.

        .. note:: You normally do not override this method, but instead
            you override :meth:`.get_target_renderer_kwargs` and
            :meth:`.get_target_renderer_class`.
        """
        return self.get_target_renderer_class()(**self.get_target_renderer_kwargs())

    def __get_target_renderer(self):
        if not hasattr(self, '_target_renderer'):
            self._target_renderer = self.get_target_renderer()
        return self._target_renderer

    def get_selectall_directive_dict(self):
        """
        Get options for the ``django-cradmin-multiselect2-selectall`` angularjs
        directive.

        Returns:
            dict: With options for the directive.
        """
        return {
            'target_dom_id': self.__get_target_renderer().get_dom_id(),
        }

    def get_selectall_directive_json(self):
        """
        Returns:
            str: The return value of :meth:`.get_select_directive_dict`
            as a json encoded and xml attribute encoded string.
        """
        return quoteattr(json.dumps(self.get_selectall_directive_dict()))

    def __has_initially_selected_items(self):
        return len(self._get_selected_values_set()) > 0

    def __select_all_button_should_be_shown(self, context):
        page_obj = context.get('page_obj', None)
        if page_obj and page_obj.number != 1:
            return False
        return self.select_all_is_allowed()

    def get_context_data(self, **kwargs):
        context = super(ViewMixin, self).get_context_data(**kwargs)
        context['target_renderer'] = self.__get_target_renderer()
        context['selectall_directive_json'] = self.get_selectall_directive_json()
        context['select_all_button_should_be_shown'] = self.__select_all_button_should_be_shown(context=context)

        # When we have initial items, we reload page1 after selecting the initially
        # selected items. This is because we need to load the items to select them,
        # but we do not want the initial selection to affect any further paging
        # and filtering on the page.
        context['must_reload_page1_on_load'] = self.__has_initially_selected_items()
        return context

    def is_initially_selected(self, value):
        """
        Should the provided value be selected on load?

        You normally do not need to override this. Override:

        - :meth:`.get_inititially_selected_queryset`
        - and for very rare cases, override :meth:`.get_postrequest_selected_queryset`.

        Args:
            value: A value in the listbuilder list (an object in the queryset).

        Returns:
            bool: If the provided value should be selected on load, return ``True``,
            otherwise return ``False``. Defaults to ``False`` if you do not overrie
            this method.
        """
        return value in self._get_selected_values_set()

    def __is_bgreplaced(self):
        return self.request.GET.get('cradmin-bgreplaced', 'false') == 'true'

    def __value_is_selected(self, value):
        if self.__is_bgreplaced():
            return False
        else:
            return self.is_initially_selected(value=value)

    def make_value_and_frame_renderer_kwargs(self, value):
        """
        This method is called by :meth:`.get_listbuilder_list_kwargs` (below)
        to create kwargs for our value and frame renderers.
        """
        return {
            'is_selected': self.__value_is_selected(value=value)
        }

    def get_value_and_frame_renderer_kwargs(self):
        """
        We return a callable here. This callable is used to create
        individual kwargs for each value in the list.
        """
        return self.make_value_and_frame_renderer_kwargs

    def get_selected_items_form_attribute(self):
        """
        Get the form attribute that contains the selected items.

        Defaults to ``"selected_items"``, so you need to override this
        if you use something else in your form.
        """
        return 'selected_items'

    def get_selectable_items_queryset(self):
        if hasattr(self, 'get_unfiltered_queryset_for_role'):
            queryset = self.get_unfiltered_queryset_for_role()
        else:
            queryset = self.get_queryset_for_role()
        return queryset

    def __make_selected_items_form_class(self):
        selected_items_form_attribute = self.get_selected_items_form_attribute()
        selectable_items_queryset = self.get_selectable_items_queryset()

        class SelectedItemsForm(forms.Form):
            def __init__(self, *args, **kwargs):
                super(SelectedItemsForm, self).__init__(*args, **kwargs)
                self.fields[selected_items_form_attribute] = forms.ModelMultipleChoiceField(
                        queryset=selectable_items_queryset
                )

        return SelectedItemsForm

    def get_postrequest_selected_queryset(self):
        """
        Get a QuerySet of the items that was selected on POST.

        If the form attribute where you set your selected items is not
        named ``"selected_items"``, you have to override :meth:`.get_selected_items_form_attribute`.

        You normally do not need to override this method.

        Optionally, you can override both :meth:`.form_invalid_init` and :meth:`.form_invalid`
        to disable autoselect of the items that was selected before post.
        """
        form_class = self.__make_selected_items_form_class()
        form = form_class(self.request.POST)
        if form.is_valid():
            selected_items_form_attribute = self.get_selected_items_form_attribute()
            return form.cleaned_data[selected_items_form_attribute]
        else:
            return self.model.objects.none()

    def get_inititially_selected_queryset(self):
        """
        Get queryset of initially selected items.

        Defaults to ``self.model.object.none()``, and you should override this
        if you want to select any objects on load.
        """
        return self.model.objects.none()

    def get_selected_values_queryset(self):
        if self.request.method == 'POST':
            queryset = self.get_postrequest_selected_queryset()
        elif self.__is_bgreplaced():
            queryset = self.model.objects.none()
        else:
            queryset = self.get_inititially_selected_queryset()
        return queryset

    def _get_selected_values_set(self):
        if not hasattr(self, '_selected_values_set'):
            self._selected_values_set = set(self.get_selected_values_queryset())
        return self._selected_values_set

    def form_invalid_init(self, form):
        """
        Called before ``form_invalid()`` to initialize the required attributes for
        ``form_invalid()``.

        If you provide a custom ``form_invalid``-method that does not re-render
        the page (I.E.: returns a HttpResponseRedirect with an error message),
        you can override this method
        """
        self.object_list = self.get_queryset()

    def form_invalid_add_global_errormessages(self, form):
        """
        Called before ``form_invalid()`` to handle adding global error messages.

        We add any error messages in the :meth:`.get_selected_items_form_attribute`
        field as a Django messages framework error message by default.

        Args:
            form: The form object.
        """
        if self.get_selected_items_form_attribute() in form.errors:
            errormessages = form.errors[self.get_selected_items_form_attribute()]
            for errormessage in errormessages:
                messages.error(self.request, errormessage)

    def post(self, request, *args, **kwargs):
        """
        Handles POST requests, instantiating a form instance with the passed
        POST variables and then checked for validity.
        """
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        else:
            self.form_invalid_init(form=form)
            self.form_invalid_add_global_errormessages(form=form)
            return self.form_invalid(form)

    def get_number_of_extra_items_in_page_with_initially_selected(self):
        """
        Get the number of extra items to add to ``paginate_by`` when
        loading initial items, and the number of initial items is larger
        than the page size.

        This is used in :meth:`.paginate_by`.

        Defaults to ``10``.
        """
        return 10

    def get_paginate_by_handling_initially_selected(self, paginate_by):
        number_of_initially_selected_items = len(self._get_selected_values_set())
        extra_items = self.get_number_of_extra_items_in_page_with_initially_selected()
        if paginate_by < (number_of_initially_selected_items + extra_items):
            # Initially select only works if the selected items is loaded,
            # so we need to override page size to ensure they are included
            paginate_by = number_of_initially_selected_items + extra_items
        return paginate_by

    def disable_paging_requested(self):
        """
        If this returns ``True``, we disable paging.

        The default implementation disables paging if ``disablePaging=true`` is
        in the querystring (in ``request.GET``).
        """
        return self.request.GET.get('disablePaging', 'false') == 'true'

    def get_default_paginate_by(self, queryset):
        """
        Get the default page size.

        See :meth:`.paginate_by` for more details.
        """
        return self.paginate_by

    def get_paginate_by(self, queryset):
        """
        Returns the page size. You should normally override :meth:`.get_default_paginate_by`
        instead of this method.

        .. warning:: Be very careful if you override this. We handle
            the following cases by default, and if you just return a static value
            here, these cases will be broken:

            - If you have initially selected items, eighter because you override
              :meth:`.get_inititially_selected_queryset` or because posting the form
              fails, and you need to re-select all the items that was posted, we ensure
              that the initial page size is the number of selected items + the number
              returned by :meth:`.get_number_of_extra_items_in_page_with_initially_selected`.
              This is handled so that the page is first loaded with this custom page size,
              and then the list is reloaded by an angularjs directive with page size set
              to :meth:`.get_default_paginate_by`.
            - If you have ``disablePaging=true`` in the querystring of the request, which
              is what the default "Select all" button does, we disable paging.

            This means that you should override:

            - :meth:`.get_default_paginate_by`
            - :meth:`.get_number_of_extra_items_in_page_with_initially_selected`

            in almost all cases unless you want to make a major rewrite of all methods
            related to handling the cases listed above.
        """
        if self.disable_paging_requested():
            return None
        paginate_by = self.get_default_paginate_by(queryset=queryset)
        if paginate_by and self.__has_initially_selected_items():
            paginate_by = self.get_paginate_by_handling_initially_selected(paginate_by=paginate_by)
        return paginate_by

    def __order_queryset(self, queryset):
        """
        Order the queryset. We have to order the queryset so that
        the initially selected items is included on the first page (if pagination is enabled).

        We do this by annotating the queryset with ``cradmin_multiselect2_ordering``,
        and inserting that as the first order_by argument.
        """
        if self.get_paginate_by(queryset) and \
                self.request.method == "POST" and self.__has_initially_selected_items():
            current_order_by = list(queryset.query.order_by)
            whenqueries = []
            max_index = 0
            for index, value in enumerate(self.get_selected_values_queryset().order_by(*current_order_by)):
                whenqueries.append(models.When(pk=value.pk, then=models.Value(index)))
                max_index = index
            queryset = queryset.annotate(
                cradmin_multiselect2_ordering=models.Case(
                    *whenqueries,
                    default=max_index + 1,
                    output_field=models.IntegerField()
                )
            )
            order_by = ['cradmin_multiselect2_ordering']
            order_by.extend(current_order_by)
            queryset = queryset.order_by(*order_by)
        return queryset

    def get_queryset(self):
        queryset = super(ViewMixin, self).get_queryset().distinct()
        queryset = self.get_selected_values_queryset().distinct() | queryset
        queryset = queryset.distinct()
        return self.__order_queryset(queryset)

    def get_total_number_of_items_in_queryset(self):
        """
        Get the total number of items in the queryset.

        .. note:: This is cached in ``self._total_number_of_items_in_queryset`` on
            first call, so calling this multiple times for a single request
            will only require one database query.
        """
        if not hasattr(self, '_total_number_of_items_in_queryset'):
            self._total_number_of_items_in_queryset = self.get_queryset().count()
        return self._total_number_of_items_in_queryset

    def get_select_all_max_items(self):
        """
        The maximum number of items to allow for select all button.

        If this returns a value other than ``None``, we do not render the "Select all" button,
        if the number of total items available for "Select all" is more than this number.

        Defaults to ``1500``.
        """
        return 1500

    def select_all_is_allowed(self):
        """
        Checks if "Select all" is allowed, and returns ``True`` if it is.

        You normally do not need to override this, override :meth:`.get_select_all_max_items`
        instead. The only reason to override this is if you want to disable "Select all"
        completely, in which case you return ``False``.
        """
        select_all_max_items = self.get_select_all_max_items()
        if select_all_max_items is None:
            return True
        else:
            return select_all_max_items > self.get_total_number_of_items_in_queryset()


class ListbuilderView(ViewMixin, listbuilderview.View):
    """
    Multiselect2 listbuilder view.
    """
    template_name = 'django_cradmin/viewhelpers/multiselect2view/listbuilderview.django.html'
    value_renderer_class = multiselect2.listbuilder_itemvalues.ItemValue


class ListbuilderFilterView(ViewMixin, listbuilderview.FilterListMixin, listbuilderview.View):
    """
    Multiselect2 listbuilder view with filters.
    """
    template_name = 'django_cradmin/viewhelpers/multiselect2view/listbuilderfilterview.django.html'
    value_renderer_class = multiselect2.listbuilder_itemvalues.ItemValue

    def get_filterlist_template_name(self):
        return self.template_name



from __future__ import unicode_literals

from django import forms

from . import container
from . import form_mixins
from . import label


class BaseFieldRenderable(container.AbstractContainerRenderable, form_mixins.FieldWrapperRenderableChildMixin):
    """
    Abstract base class for renders of the actual form field
    for a :class:`.FieldWrapper`.

    You never use this on its own outside a :class:`.FieldWrapper`.
    """
    def __init__(self, autofocus=False, placeholder=False, **kwargs):
        """

        Args:
            autofocus (boolean): Autofocus on this field at page load?
            placeholder (str): Placeholder text. See
                :meth:`~django_cradmin.uicontainer.container.AbstractContainerRenderable.get_html_element_attributes`
                for details about how values are applied.
            **kwargs: Kwargs for :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self._overridden_autofocus = autofocus
        self._overridden_placeholder = placeholder
        super(BaseFieldRenderable, self).__init__(**kwargs)

    def get_default_autofocus(self):
        """
        Get the default value for the autofocus attribute of the html element.

        Defaults to ``False``.
        """
        return False

    @property
    def autofocus(self):
        """
        Get the value for the autofocus attribute of the html element.

        You should not override this. Override :meth:`.get_default_autofocus` instead.
        """
        return self._overridden_autofocus or self.get_default_autofocus()

    def get_default_placeholder(self):
        """
        Get the default value for the placeholder attribute of the html element.

        Defaults to ``False``.
        """
        return False

    @property
    def placeholder(self):
        """
        Get the value for the placeholder attribute of the html element.

        You should not override this. Override :meth:`.get_default_placeholder` instead.
        """
        return self._overridden_placeholder or self.get_default_placeholder()

    def get_html_element_attributes(self):
        attributes = super(BaseFieldRenderable, self).get_html_element_attributes()
        attributes.update({
            'autofocus': self.autofocus,
            'placeholder': self.placeholder,
        })
        return attributes

    def should_have_for_attribute_on_label(self):
        """
        Should this field get a ``for`` attribute on its label?

        Used by :class:`django_cradmin.uicontainer.label.Label`.

        Defaults to ``True``, but subclasses can override this.
        """
        return True

    def should_render_as_child_of_label(self):
        """
        Should this field be renderered as a child of the ``<label>``?

        Returns ``True`` by default, but subclasses should override this
        for fields that should not be rendered within a label.

        For example, :class:`.Field` overrides this with varying behavior
        depending on the type of the Django field widget.
        """
        return True

    @property
    def bound_formfield(self):
        return self.field_wrapper_renderable.formrenderable.form[self.field_wrapper_renderable.fieldname]

    def get_default_dom_id(self):
        return self.bound_formfield.auto_id

    @property
    def field_attributes_dict(self):
        attributes_dict = {}
        attributes_dict = self.bound_formfield.build_widget_attrs(attributes_dict)
        attributes_dict.update(self.get_html_element_attributes())
        return attributes_dict


class SubWidgetField(BaseFieldRenderable):
    template_name = 'django_cradmin/uicontainer/field/subwidget.django.html'

    def __init__(self, django_subwidget, index_in_parent, **kwargs):
        self.django_subwidget = django_subwidget
        self.index_in_parent = index_in_parent
        super(SubWidgetField, self).__init__(**kwargs)

    @property
    def label_text(self):
        return self.django_subwidget.choice_label

    def can_have_children(self):
        return False

    @property
    def field_attributes_dict(self):
        attributes_dict = {}
        attributes_dict.update(self.django_subwidget.attrs)
        attributes_dict.update(self.get_html_element_attributes())
        if self.dom_id:
            attributes_dict['id'] = '{dom_id}_{index_in_parent}'.format(
                dom_id=self.dom_id,
                index_in_parent=self.index_in_parent)
        return attributes_dict

    @property
    def rendered_subwidget(self):
        return self.django_subwidget.tag(attrs=self.field_attributes_dict)


class Field(BaseFieldRenderable):
    """
    Automatically renders a form field for a :class:`.FieldWrapper`
    using the Django widget system.

    You never use this on its own outside a :class:`.FieldWrapper`.

    Examples:

        First of all, this is always used within a
        :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`,
        and by default FieldWrapper uses this class without any kwargs
        as the ``field_renderable``. So the following will render a
        field on the form named ``email`` no matter what the type or
        widget of the field is::

            from django_cradmin import uicontainer

            class ExampleForm(forms.Form):
                email = forms.EmailField()

            # You would put the fieldwrapper as a child somewhere below
            # a ``django_cradmin.uicontainer.form.Form``, but we
            # do not include that in these examples for brevity. See the docs for
            # ``django_cradmin.uicontainer.form.Form`` for a complete example.
            uicontainer.fieldwrapper.FieldWrapper(fieldname='email')

        Let us adjust some parameters of the field rendered by the FieldWrapper
        by providing a custom field_renderable::

            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='email',
                field_wrapper_renderable=uicontainer.field.Field(
                    autofocus=True,  # Autofocus on page load
                    placeholder="Type your email...",  # Set a placeholder text

                    # Change the css classes - the css class of the field will
                    # be ``custom-input custom-input--big``
                    bem_block='custom-input',
                    bem_variant_list=['big']

                    # Alternatively, we can set the css class
                ))

        What about some CSS classes on the input field? We can do that using
        BEM naming, by specifying css classes directly as a list,
        or by just adding some css classes. Lets look at the BEM options::

            # Change the BEM block from ``input`` (the default) to ``custom-input``.
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='email',
                field_wrapper_renderable=uicontainer.field.Field(
                    bem_block='custom-input',
                    bem_variant_list=['big']
                ))

            # .. or just add some variants ..
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='email',
                field_wrapper_renderable=uicontainer.field.Field(
                    bem_variant_list=['inline', 'small']
                ))

            # .. or both ..
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='email',
                field_wrapper_renderable=uicontainer.field.Field(
                    bem_block='custom-input',
                    bem_variant_list=['large']
                ))

            # In all cases, the resulting CSS class will be
            # <bem_block> [<bem_block>--<variant0]  [<bem_block>--<variantN>]

        Refer to :ref:`uicontainer_styling` for more details about setting
        CSS classes.

        Field just works automatically with choice fields, but that uses a
        ``<select>`` by default. Let us render a ChoiceField as a
        radio button list instead::

            class ExampleForm(forms.Form):
                user_type = forms.ChoiceField(
                    choices=[
                        ('standard', 'Standard'),
                        ('admin', 'Admin'),
                    ],
                    initial='standard',
                    widget=forms.RadioSelect()
                )

            # Nothing special is needed - we pick up that you override
            # the widget on the form field, and just render it differently.
            uicontainer.fieldwrapper.FieldWrapper(fieldname='user_type')

            # We can override options for the sub-widgets (the radio buttons)
            # Lets change the CSS class of the label around each of the radio buttons
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='user_type',
                field_renderable=uicontainer.field.Field(
                    subwidget_renderable_kwargs={
                        'bem_variant_list': ['inline']
                    }
                )
            )

        CheckboxSelectMultiple widgets work just like the RadioSelect widget.

        If you need more customization than the ``subwidget_renderable_kwargs`` option
        provides, you can customize exactly how RadioSelect and CheckboxSelectMultiple is rendered
        by extending :class:`.Field` and override:

        - :meth:`~.Field.make_subwidget_renderable_radio`
        - :meth:`~.Field.make_subwidget_renderable_checkbox`

    """
    template_name = 'django_cradmin/uicontainer/field/field.django.html'

    def __init__(self, subwidget_renderable_kwargs=None, **kwargs):
        """

        Args:
            subwidget_renderable_kwargs: Kwargs for the subwidget renderable.
                See :meth:`.make_subwidget_renderable`. You can provide any
                kwargs that works with
                :class:`~django_cradmin.uicontainer.container.AbstractContainerRenderable`.
            **kwargs: Kwargs for :class:`.BaseFieldRenderable`.
        """
        self.extra_subwidget_renderable_kwargs = subwidget_renderable_kwargs or {}
        super(Field, self).__init__(**kwargs)
        self.properties['field_wrapper_renderable'] = self

    @property
    def django_widget(self):
        """
        Get the Django Widget object for the formfield.
        """
        return self.bound_formfield.field.widget

    def is_radio_select_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.RadioSelect`.
        """
        return isinstance(self.django_widget, forms.RadioSelect)

    def is_checkbox_select_multiple_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.CheckboxSelectMultiple`.
        """
        return isinstance(self.django_widget, forms.CheckboxSelectMultiple)

    def is_select_multiple_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.SelectMultiple`.
        """
        return isinstance(self.django_widget, forms.SelectMultiple)

    def is_select_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.Select`.
        """
        return isinstance(self.django_widget, forms.Select)

    def should_render_as_subwidgets(self):
        """
        Returns True if we should render using :meth:`.make_subwidget_renderable`.

        Default to returining ``True`` if :meth:`.is_radio_select_widget` or
        :meth:`.is_checkbox_select_multiple_widget` returns ``True``.
        """
        return (self.is_radio_select_widget()
                or self.is_checkbox_select_multiple_widget())

    def should_have_for_attribute_on_label(self):
        """
        Should this field get a ``for`` attribute on its label?

        Used by :class:`django_cradmin.uicontainer.label.Label`.

        Defaults to :meth:`.should_render_as_subwidgets`.
        """
        return not self.should_render_as_subwidgets()

    def should_render_as_child_of_label(self):
        """
        Should this field be rendered as a child of the label?

        Used by :class:`django_cradmin.uicontainer.label.Label`.
        """
        return not (self.should_render_as_subwidgets()
                    or self.is_select_widget()
                    or self.is_select_multiple_widget())

    def get_default_bem_block_or_element(self):
        """
        Default BEM block is ``input``.
        """
        if self.is_select_widget():
            return None
        else:
            return 'input'

    def get_default_test_css_class_suffixes_list(self):
        return super(Field, self).get_default_test_css_class_suffixes_list() + [
            'djangowidget-{}'.format(self.django_widget.__class__.__name__.lower())
        ]

    def render_bound_formfield_as_widget(self):
        return self.bound_formfield.as_widget(attrs=self.field_attributes_dict)

    def get_subwidget_renderable_kwargs(self, django_subwidget, index_in_parent):
        """
        Get kwargs for the subwidget created by :meth:`.make_subwidget_renderable`.

        Defaults to the ``subwidget_renderable_kwargs`` kwargs + adding
        a :class:`.SubWidgetField` object as ``subwidget_field_renderable``.
        """
        kwargs = dict(self.extra_subwidget_renderable_kwargs)
        kwargs.update({
            'subwidget_field_renderable': SubWidgetField(django_subwidget=django_subwidget,
                                                         index_in_parent=index_in_parent)
        })
        return kwargs

    def make_subwidget_renderable_radio(self, django_subwidget, index_in_parent):
        """
        Make subwidget renderable to use if :meth:`.is_radio_select_widget` returns ``True``.

        Returns a :class:`django_cradmin.uicontainer.label.RadioSubWidgetLabel`
        with kwargs from :meth:`.get_subwidget_renderable_kwargs` by default.

        Used by :meth:`.make_subwidget_renderable`.

        Args:
            django_subwidget: The SubWidget object.
            index_in_parent: The index of the subwidget in the parent.
        """
        return label.RadioSubWidgetLabel(**self.get_subwidget_renderable_kwargs(
            django_subwidget=django_subwidget,
            index_in_parent=index_in_parent
        ))

    def make_subwidget_renderable_checkbox(self, django_subwidget, index_in_parent):
        """
        Make subwidget renderable to use if :meth:`.is_checkbox_select_multiple_widget` returns ``True``.

        Returns a :class:`django_cradmin.uicontainer.label.CheckboxSubWidgetLabel`
        with kwargs from :meth:`.get_subwidget_renderable_kwargs` by default.

        Used by :meth:`.make_subwidget_renderable`.

        Args:
            django_subwidget: The SubWidget object.
            index_in_parent: The index of the subwidget in the parent.
        """
        return label.CheckboxSubWidgetLabel(**self.get_subwidget_renderable_kwargs(
            django_subwidget=django_subwidget,
            index_in_parent=index_in_parent
        ))

    def make_subwidget_renderable(self, django_subwidget, index_in_parent):
        """
        Make a renderable for a Django SubWidget.

        Only used if :meth:`.should_render_as_subwidgets` returns ``True``.

        By default this returns:

        - :meth:`.make_subwidget_renderable_radio` if :meth:`.is_radio_select_widget` returns ``True``.
        - :meth:`.make_subwidget_renderable_checkbox` if :meth:`.is_checkbox_select_multiple_widget` returns ``True``.
        - Raise NotImplementedError otherwise.

        Args:
            django_subwidget: The SubWidget object.
            index_in_parent: The index of the subwidget in the parent.

        """
        if self.is_radio_select_widget():
            return self.make_subwidget_renderable_radio(
                django_subwidget=django_subwidget, index_in_parent=index_in_parent)
        elif self.is_checkbox_select_multiple_widget():
            return self.make_subwidget_renderable_checkbox(
                django_subwidget=django_subwidget, index_in_parent=index_in_parent)
        else:
            widget_class = self.django_widget.__class__
            raise NotImplementedError(
                'The {widget_class!r} widget is not supported.'.format(
                    widget_class='{}.{}'.format(widget_class.__module__, widget_class.__name__)
                ))

    def iter_subwidgets(self):
        for index, django_subwidget in enumerate(self.django_widget.subwidgets(
                name=self.bound_formfield.html_name,
                value=self.bound_formfield.value(),
                attrs=self.field_attributes_dict)):
            subwidget = self.make_subwidget_renderable(
                django_subwidget=django_subwidget,
                index_in_parent=index)
            subwidget.bootstrap(parent=self)
            yield subwidget

    @property
    def rendered_field(self):
        return self.render_bound_formfield_as_widget()


class HiddenField(Field):
    """
    Just like :class:`.Field`, but renders as
    a ``<input type="hidden">``.
    """
    @property
    def rendered_field(self):
        return self.bound_formfield.as_hidden(attrs=self.field_attributes_dict)



from django.conf import settings


def get_setting(setting, fallbackvalue=None):
    """
    Get a Django setting falling back to the given ``fallbackvalue``
    if ``settings.<setting>`` is not set.
    """
    return getattr(settings, setting, fallbackvalue)



class FieldWrapperRenderableChildMixin(object):
    """
    Mixin class for renderables that are children of :class:`.FieldWrapper`.
    """
    @property
    def field_wrapper_renderable(self):
        return self.properties['field_wrapper_renderable']


class FieldChildMixin(FieldWrapperRenderableChildMixin):
    """
    Mixin class for renderables that are children of :class:`.FieldWrapper`.
    """
    @property
    def field_renderable(self):
        return self.properties['field_renderable']


class FormRenderableChildMixin(object):
    @property
    def formrenderable(self):
        return self.properties['formrenderable']



from django.conf import settings
from django.contrib.auth.views import logout


def cradmin_logoutview(request, template_name='cradmin_authenticate/logout.django.html'):
    next_page = None
    if 'next' in request.GET:
        next_page = request.GET['next']
    return logout(
        request,
        template_name=template_name,
        next_page=next_page,
        extra_context={
            'LOGIN_URL': settings.LOGIN_URL,
            'cradmin_javascriptregistry_component_ids': []
        })
